from .auto_delete_message import AutoDeleteMessage
from .create_poll import CreatePoll
from .delete_messages import DeleteMessages
from .edit_message import EditMessage
from .forward_messages import ForwardMessages
from .get_messages_by_id import GetMessagesByID
from .get_messages_interval import GetMessagesInterval
from .get_poll_option_voters import GetPollOptionVoters
from .get_poll_status import GetPollStatus
from .request_send_file import RequestSendFile
from .send_message import SendMessage
from .send_document import SendDocmuent
from .send_gif import SendGif
from .send_music import SendMusic
from .send_video import SendVideo
from .send_voice import SendVoice
from .send_photo import SendPhoto
from .set_pin_message import SetPinMessage
from .vote_poll import VotePoll
from .send_video_message import SendVideoMessage
from .action_on_message_reaction import ActionOnMessageReaction
from .reaction import Reaction
from .remove_reaction import RemoveReaction
from .get_message_share_url import GetMessageShareUrl
from .get_messages_updates import GetMessagesUpdates


class Messages(
        AutoDeleteMessage,
        CreatePoll,
        DeleteMessages,
        EditMessage,
        ForwardMessages,
        GetMessagesByID,
        GetMessagesInterval,
        GetPollOptionVoters,
        GetPollStatus,
        RequestSendFile,
        SendMessage,
        SetPinMessage,
        VotePoll,
        SendDocmuent,
        SendGif,
        SendMusic,
        SendVideo,
        SendVoice,
        SendPhoto,
        SendVideoMessage,
        ActionOnMessageReaction,
        RemoveReaction,
        Reaction,
        GetMessageShareUrl,
        GetMessagesUpdates,
):
        pass


from typing import Literal
from pydantic import BaseModel


class ForwardedFrom(BaseModel):
    type_from: Literal['User', 'Channel', 'Bot']
    message_id: str
    from_chat_id: str
    from_sender_id: str


from typing import Optional


class GetChats:
    async def get_chats(
            self,
            start_id: Optional[str]=None,
    ):
        return await self.builder('getChats',
                                  input={
                                      'start_id': start_id,
                                  })


from .aux_data import AuxData
from .bot import Bot
from .bot_command import BotCommand
from .button import Button
from .button_calendar import ButtonCalendar
from .button_location import ButtonLocation
from .button_number_picker import ButtonNumberPicker
from .button_selection import ButtonSelection
from .button_selection_item import ButtonSelectionItem
from .button_string_picker import ButtonStringPicker
from .button_textbox import ButtonTextbox
from .chat import Chat
from .contact_message import ContactMessage
from .file import File
from .forwarded_from import ForwardedFrom
from .inline_message import InlineMessage
from .keypad import Keypad
from .keypad_row import KeypadRow
from .live_location import LiveLocation
from .location import Location
from .messaage_text_update import MessaageTextUpdate
from .message import Message
from .message_keypad_update import MessageKeypadUpdate
from .payment_status import PaymentStatus
from .poll import Poll
from .poll_status import PollStatus
from .sticker import Sticker
from .update import Update


from typing import Optional
import rubpy


class SendCode:
    async def send_code(self: "rubpy.Client",
                        phone_number: str,
                        pass_key: Optional[str] = None,
                        send_type: Optional[str] = 'SMS',
    ):
        if send_type not in ('SMS', 'Internal'):
            raise ValueError('send_type can only be `SMS` or `Internal`.')

        data = {'phone_number': phone_number,
                'pass_key': pass_key,
                'send_type': send_type}

        return await self.builder(name='sendCode',
                                 input=data,
                                 tmp_session=True,)


import rubpy

class ChannelPreviewByJoinLink:
    async def channel_preview_by_join_link(
            self: "rubpy.Client",
            link: str,
    ):
        if '/' in link:
            link = link.split('/')[-1]

        return await self.builder('channelPreviewByJoinLink',
                                  input={
                                      'hash_link': link,
                                  })


class SetGroupVoiceChatSetting:
    async def set_group_voice_chat_setting(
            self,
            group_guid: str,
            voice_chat_id: str,
            title: str=None,
    ):
        input = {
            'group_guid': group_guid,
            'voice_chat_id': voice_chat_id,
        }
        updated_parameters = []

        if title is not None:
            input['title'] = title
            updated_parameters.append('title')

        return await self.builder('setGroupVoiceChatSetting',
                                  input=input)


class GetUserInfo:
    async def get_user_info(
            self,
            user_guid: str=None,
    ):
        result = await self.builder(
            name='getUserInfo',
            input={} if user_guid is None else {'user_guid': user_guid},
            tmp_session=True if self.auth is None else False,
        )
        return result


from typing import Literal, Optional
from pydantic import BaseModel
from .button_selection import ButtonSelection
from .button_calendar import ButtonCalendar
from .button_number_picker import ButtonNumberPicker
from .button_string_picker import ButtonStringPicker
from .button_location import ButtonLocation
from .button_textbox import ButtonTextbox


class Button(BaseModel):
    id: str
    type: Literal['Simple', 'Selection', 'Calendar', 'NumberPicker', 'StringPicker', 'Location', 'Payment',
                  'CameraImage', 'CameraVideo', 'GalleryImage', 'GalleryVideo', 'File', 'Audio', 'RecordAudio',
                  'MyPhoneNumber', 'MyLocation', 'Textbox', 'Link', 'AskMyPhoneNumber', 'AskLocation', 'Barcode']
    button_text: str
    button_selection: Optional[ButtonSelection]
    button_calendar: Optional[ButtonCalendar]
    button_number_picker: Optional[ButtonNumberPicker]
    button_string_picker: Optional[ButtonStringPicker]
    button_location: Optional[ButtonLocation]
    button_textbox: Optional[ButtonTextbox]
    # button_link: Optional[Link]


import rubpy

class SetChannelLink:
    async def set_channel_link(
            self: "rubpy.Client",
            channel_guid: str,
    ):
        return await self.builder('setChannelLink',
                                  input={
                                      'channel_guid': channel_guid,
                                  })


class GetObjectByUsername:
    async def get_object_by_username(
            self,
            username: str,
    ):
        username = username.replace('@', '')
        return await self.builder('getObjectByUsername',
                                  input={
                                      'username': username,
                                  })


from .sessions import SQLiteSession, StringSession
from .client import Client
from .rubino import Rubino
from . import types, utils, filters, exceptions, enums

__author__ = 'Shayan Heidari' # Forked by amirali irvany
__version__ = '0.0.2'



import rubpy

class GetChannelInfo:
    async def get_channel_info(
            self: "rubpy.Client",
            channel_guid: str,
    ):
        return await self.builder('getChannelInfo',
                                  input={
                                      'channel_guid': channel_guid,
                                  })


import rubpy

class GetMessageShareUrl:
    async def get_message_url(
            self: "rubpy.Client",
            object_guid: str,
            message_id: str,
    ):
        input = dict(
            object_guid=object_guid,
            message_id=message_id,
        )

        return await self.builder(name='getMessageShareUrl',
                                  input=input)


class SearchGlobalObjects:
    async def search_global_objects(
            self,
            search_text: str,
    ):
        return await self.builder('searchGlobalObjects',
                                  input={
                                      'search_text': search_text,
                                  })


from typing import Type
import difflib
import inspect
import warnings
import sys
import re

__all__ = ['Operator', 'BaseModel', 'RegexModel']
__models__ = [
    'is_pinned', 'is_mute', 'count_unseen', 'message_id',
    'is_group', 'is_private', 'is_channel', 'is_in_contact',
    'raw_text', 'original_update', 'object_guid', 'author_guid', 'time', 'reply_message_id']

def create(name, __base, authorise: list = [], exception: bool = True, *args, **kwargs):
        result = None
        if name in authorise:
            result = name

        else:
            proposal = difflib.get_close_matches(name, authorise, n=1)
            if proposal:
                result = proposal[0]
                caller = inspect.getframeinfo(inspect.stack()[2][0])
                warnings.warn(
                    f'{caller.filename}:{caller.lineno}: do you mean'
                    f' "{name}", "{result}"? correct it')

        if result is not None or not exception:
            if result is None:
                result = name
            return type(result, __base, {'__name__': result, **kwargs})

        raise AttributeError(f'module has no attribute ({name})')


class Operator:
    Or = 'OR'
    And = 'AND'
    Less = 'Less'
    Lesse = 'Lesse'
    Equal = 'Equal'
    Greater = 'Greater'
    Greatere = 'Greatere'
    Inequality = 'Inequality'

    def __init__(self, value, operator, *args, **kwargs):
        self.value = value
        self.operator = operator

    def __eq__(self, value) -> bool:
        return self.operator == value


class BaseModel:
    def __init__(self, func=None, filters=[], *args, **kwargs) -> None:
        self.func = func
        if not isinstance(filters, list):
            filters = [filters]
        self.filters = filters

    def insert(self, filter):
        self.filters.append(filter)
        return self

    def __or__(self, value):
        return self.insert(Operator(value, Operator.Or))

    def __and__(self, value):
        return self.insert(Operator(value, Operator.And))

    def __eq__(self, value):
        return self.insert(Operator(value, Operator.Equal))

    def __ne__(self, value):
        return self.insert(Operator(value, Operator.Inequality))

    def __lt__(self, value):
        return self.insert(Operator(value, Operator.Less))

    def __le__(self, value):
        return self.insert(Operator(value, Operator.Lesse))

    def __gt__(self, value):
        return self.insert(Operator(value, Operator.Greater))

    def __ge__(self, value):
        return self.insert(Operator(value, Operator.Greatere))

    async def build(self, update):
        # get key
        result = getattr(update, self.__class__.__name__, None)
        if callable(self.func):
            if update.is_async(self.func):
                result = await self.func(result)
            else:
                result = self.func(result)

        for filter in self.filters:
            value = filter.value

            # if the comparison was with a function
            if callable(value):
                if update.is_async(value):
                    value = await value(update, result)
                else:
                    value = value(update, result)

            if self.func:
                if update.is_async(self.func):
                    value = await self.func(value)
                else:
                    value = self.func(value)

            if filter == Operator.Or:
                result = result or value

            elif filter == Operator.And:
                result = result and value

            elif filter == Operator.Less:
                result = result < value

            elif filter == Operator.Lesse:
                result = result <= value

            elif filter == Operator.Equal:
                result = result == value

            elif filter == Operator.Greater:
                result = result > value

            elif filter == Operator.Greatere:
                result = result >= value

            elif filter == Operator.Inequality:
                result = result != value

        return bool(result)

    async def __call__(self, update, *args, **kwargs):
        return await self.build(update)


class RegexModel(BaseModel):
    def __init__(self, pattern: str, *args, **kwargs) -> None:
        self.pattern = re.compile(pattern)
        super().__init__(*args, **kwargs)

    async def __call__(self, update, *args, **kwargs) -> bool:
        if update.raw_text is None:
            return False

        update.pattern_match = self.pattern.match(update.raw_text)
        return bool(update.pattern_match)


class Models:
    def __init__(self, name, *args, **kwargs) -> None:
        self.__name__ = name

    def __eq__(self, value: object) -> bool:
        return BaseModel in value.__bases__

    def __dir__(self):
        return sorted(__models__)

    def __call__(self, name, *args, **kwargs):
        return self.__getattr__(name)

    def __getattr__(self, name):
        if name in __all__:
            return globals()[name]
        return create(name, (BaseModel,), authorize=__models__, exception=False)

    # async def text(self):
    #     print('call text')
    #     return create('text', (BaseModel,), exception=False)

    # async def text(self, *args, **kwargs):
    #     print('called.')
    #     return Operator('text', Operator.Equal)

sys.modules[__name__] = Models(__name__)

is_pinned: Type[BaseModel]
is_mute: Type[BaseModel]
count_unseen: Type[BaseModel]
message_id: Type[BaseModel]
is_group: Type[BaseModel]
is_private: Type[BaseModel]
is_channel: Type[BaseModel]
is_in_contact: Type[BaseModel]
raw_text: Type[BaseModel]
original_update: Type[BaseModel]
object_guid: Type[BaseModel]
author_guid: Type[BaseModel]
time: Type[BaseModel]
reply_message_id: Type[BaseModel]


from typing import Optional, Union
from pathlib import Path
import rubpy

class SendDocmuent:
    async def send_document(
                self: "rubpy.Client",
                object_guid: str,
                document: Union[Path, bytes],
                caption: Optional[str] = None,
                reply_to_message_id:Optional[str] = None,
                auto_delete: Optional[int]=None, *args, **kwargs,
    ):
        """
        Send a document.

        Args:
            object_guid (str):
                The GUID of the recipient.

            document (bytes):
                The document data.

            caption (str, optional):
                The caption for the document. Defaults to None.

            reply_to_message_id (str, optional):
                The ID of the message to which this is a reply. Defaults to None.
        """
        return await self.send_message(object_guid=object_guid, text=caption, reply_to_message_id=reply_to_message_id, file_inline=document, thumb=False, auto_delete=auto_delete, *args, **kwargs)


from typing import Union, Optional
from pathlib import Path
import rubpy

class SendVoice:
    async def send_voice(
            self: "rubpy.Client",
            object_guid: str,
            voice: Union[Path, bytes],
            caption: Optional[str] = None,
            reply_to_message_id: Optional[str] = None,
            auto_delete: Optional[int] = None, *args, **kwargs,
    ):
        """
        Send a voice.

        Args:
            object_guid (str):
                The GUID of the recipient.

            voice (Path, bytes):
                The voice data.

            caption (str, optional):
                The caption for the voice. Defaults to None.

            reply_to_message_id (str, optional):
                The ID of the message to which this is a reply. Defaults to None.
        """

        return await self.send_message(object_guid=object_guid, text=caption, reply_to_message_id=reply_to_message_id, file_inline=voice, type='Voice', auto_delete=auto_delete, *args, **kwargs)


from pydantic import BaseModel
from typing import List, Optional

class OnlineTime(BaseModel):
    type: Optional[str] = None
    exact_time: Optional[int] = None

class LastMessage(BaseModel):
    message_id: Optional[str] = None
    type: Optional[str] = None
    text: Optional[str] = None
    author_object_guid: Optional[str] = None
    is_mine: Optional[bool] = None
    author_title: Optional[str] = None
    author_type: Optional[str] = None

class AbsObject(BaseModel):
    object_guid: Optional[str] = None
    type: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None

class Chat(BaseModel):
    object_guid: Optional[str] = None
    access: Optional[List[str]] = None
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[LastMessage] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None
    abs_object: Optional[AbsObject] = None
    is_blocked: Optional[bool] = None
    last_message_id: Optional[str] = None
    last_deleted_mid: Optional[str] = None

class UserInfo(BaseModel):
    user_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    phone: Optional[str] = None
    username: Optional[str] = None
    last_online: Optional[int] = None
    bio: Optional[str] = None
    is_deleted: Optional[bool] = None
    is_verified: Optional[bool] = None
    online_time: Optional[OnlineTime] = None

class GetUserInfo(BaseModel):
    user: Optional[UserInfo] = None
    chat: Optional[Chat] = None
    timestamp: Optional[str] = None
    can_receive_call: Optional[bool] = None
    can_video_call: Optional[bool] = None

class ChatAccess(BaseModel):
    access: Optional[List[str]] = None
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[LastMessage] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None
    pinned_message_id: Optional[str] = None
    abs_object: Optional[AbsObject] = None
    is_blocked: Optional[bool] = None
    last_message_id: Optional[str] = None
    last_deleted_mid: Optional[str] = None
    slow_mode_duration: Optional[int] = None
    group_my_last_send_time: Optional[int] = None
    pinned_message_ids: Optional[List[str]] = None

class ChatUpdate(BaseModel):
    object_guid: Optional[str] = None
    action: Optional[str] = None
    chat: Optional[ChatAccess] = None
    updated_parameters: Optional[List[str]] = None
    timestamp: Optional[str] = None
    type: Optional[str] = None

class BlockUser(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class DeleteUserChat(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class CheckUserName(BaseModel):
    exist: Optional[bool] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None



from typing import Optional
from pydantic import BaseModel
from .avatar_thumbnail import AvatarThumbnail

class AbsObject(BaseModel):
    object_guid: Optional[str] = None
    type: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None


class SetActionChat:
    async def set_action_chat(
            self,
            object_guid: str,
            action: str='Mute',
    ):
        if action not in ('Mute', 'Unmute'):
            raise ValueError('`action` argument can only be `Mute` or `Unmute`.')
    
        return await self.builder('setActionChat',
                                  input={
                                      'object_guid': object_guid,
                                      'action': action,
                                  })


from pydantic import BaseModel
from typing import Optional, List, Dict, Union

class LastMessage(BaseModel):
    message_id: Optional[str] = None
    type: Optional[str] = None
    text: Optional[str] = None
    author_object_guid: Optional[str] = None
    is_mine: Optional[bool] = None
    author_title: Optional[str] = None
    author_type: Optional[str] = None

class ChatAccess(BaseModel):
    access: Optional[List[str]] = []
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[LastMessage] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None
    pinned_message_id: Optional[str] = None
    abs_object: Optional[Dict] = None
    is_blocked: Optional[bool] = None
    last_message_id: Optional[str] = None
    last_deleted_mid: Optional[str] = None
    slow_mode_duration: Optional[int] = None
    group_my_last_send_time: Optional[int] = None
    pinned_message_ids: Optional[List[str]] = []

class ChatUpdate(BaseModel):
    object_guid: Optional[str] = None
    action: Optional[str] = None
    chat: Optional[ChatAccess] = None
    updated_parameters: Optional[List[str]] = None
    timestamp: Optional[str] = None
    type: Optional[str] = None

class LeaveGroup(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class Group(BaseModel):
    group_guid: Optional[str] = None
    group_title: Optional[str] = None
    count_members: Optional[int] = None
    is_deleted: Optional[bool] = None
    is_verified: Optional[bool] = None
    slow_mode: Optional[int] = None
    chat_history_for_new_members: Optional[str] = None
    event_messages: Optional[bool] = None
    chat_reaction_setting: Optional[Union[str, Dict]] = None

class Chat(BaseModel):
    object_guid: Optional[str] = None
    action: Optional[str] = None
    access: Optional[List[str]] = None
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[Dict] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None
    abs_object: Optional[Dict] = None
    is_blocked: Optional[bool] = None
    last_message_id: Optional[str] = None
    last_deleted_mid: Optional[str] = None
    slow_mode_duration: Optional[int] = None

class MessageUpdate(BaseModel):
    message_id: Optional[str] = None
    action: Optional[str] = None
    message: Optional[Dict] = None
    updated_parameters: Optional[List[str]] = None
    timestamp: Optional[str] = None
    prev_message_id: Optional[str] = None
    object_guid: Optional[str] = None
    type: Optional[str] = None
    state: Optional[str] = None

class JoinGroup(BaseModel):
    group: Optional[Group] = None
    is_valid: Optional[bool] = None
    chat_update: Optional[Chat] = None
    message_update: Optional[MessageUpdate] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class AddGroup(BaseModel):
    group: Optional[Group] = None
    chat_update: Optional[Chat] = None
    message_update: Optional[MessageUpdate] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class RemoveGroup(BaseModel):
    chat_update: Optional[Chat] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetGroupInfo(BaseModel):
    group: Optional[Group] = None
    chat: Optional[Chat] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None
    message_update: Optional[MessageUpdate] = None

class GroupLink(BaseModel):
    join_link: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class EditGroupInfo(BaseModel):
    group: Optional[Group] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class OnlineTime(BaseModel):
    type: Optional[str] = None
    approximate_period: Optional[str] = None

class InChatMemberList(BaseModel):
    member_type: Optional[str] = None
    member_guid: Optional[str] = None
    first_name: Optional[str] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None
    promoted_by_object_guid: Optional[str] = None
    promoted_by_object_type: Optional[str] = None
    join_type: Optional[str] = None
    online_time: Optional[OnlineTime] = None

class InChatMember(BaseModel):
    in_chat_member: Optional[InChatMemberList] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class BanGroupMember(BaseModel):
    group: Optional[Group] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[Dict] = None

class AvatarThumbnail(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None

class AddedInChatMember(BaseModel):
    member_type: Optional[str] = None
    member_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None
    last_online: Optional[int] = None
    join_type: Optional[str] = None
    username: Optional[str] = None
    online_time: Optional[OnlineTime] = None

class AddGroupMembers(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    message_update: Optional[MessageUpdate] = None
    added_in_chat_members: Optional[List[AddedInChatMember]] = None
    timestamp: Optional[str] = None
    group: Optional[Group] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class InChatGroupMember(BaseModel):
    member_type: Optional[str] = None
    member_guid: Optional[str] = None
    first_name: Optional[str] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None
    last_online: Optional[int] = None
    join_type: Optional[str] = None
    online_time: Optional[OnlineTime] = None
    promoted_by_object_guid: Optional[str] = None
    promoted_by_object_type: Optional[str] = None
    removed_by_object_guid: Optional[str] = None
    removed_by_object_type: Optional[str] = None

class GetAllGroupMembers(BaseModel):
    in_chat_members: Optional[List[InChatGroupMember]] = None
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetGroupAdminMembers(BaseModel):
    in_chat_members: Optional[List[InChatGroupMember]] = None
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetGroupMentionList(BaseModel):
    in_chat_members: Optional[List[InChatGroupMember]] = None
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetGroupDefaultAccess(BaseModel):
    access_list: Optional[List] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class OnlineTime(BaseModel):
    type: Optional[str] = None
    approximate_period: Optional[str] = None

class TopParticipants(BaseModel):
    member_type: Optional[str] = None
    member_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None
    username: Optional[str] = None
    online_time: Optional[OnlineTime] = None

class GroupPreviewByJoinLink(BaseModel):
    is_valid: Optional[bool] = None
    group: Optional[Group] = None
    has_joined: Optional[bool] = None
    top_participants: Optional[List[TopParticipants]] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class DeleteNoAccessGroupChat(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    status: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetGroupAdminAccessList(BaseModel):
    access_list: Optional[List] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetBannedGroupMembers(BaseModel):
    in_chat_members: Optional[List[InChatGroupMember]] = []
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None


import base64
import typing
import tempfile
import warnings
import io

try:
    import cv2
    import numpy as np

except ImportError as e:
    cv2 = None
    numpy = None

try:
    import PIL.Image

except ImportError as e:
    PIL = None


class ResultMedia:
    def __repr__(self) -> str:
        return repr(vars(self))

    def __init__(self,
                 image: bytes,
                 width: typing.Optional[int] = 200,
                 height: typing.Optional[int] = 200,
                 seconds: typing.Optional[int] = 1) -> None:
        self.image = image
        self.width = width
        self.height = height
        self.seconds = seconds

        if hasattr(cv2, 'imdecode'):
            if not isinstance(image, np.ndarray):
                image = np.frombuffer(image, dtype=np.uint8)
                image = cv2.imdecode(image, flags=1)

            self.image = self.ndarray_to_bytes(image)

    def ndarray_to_bytes(self, image, *args, **kwargs) -> str:
        if hasattr(cv2, 'resize'):
            self.width = image.shape[1]
            self.height = image.shape[0]
            image = cv2.resize(image,
                               (round(self.width / 10), round(self.height / 10)),
                               interpolation=cv2.INTER_CUBIC)

            status, buffer = cv2.imencode('.png', image)
            if status is True:
                return io.BytesIO(buffer).read()

    def to_base64(self):
        return base64.b64encode(self.image).decode('utf-8')


class MediaThumbnail:
    @classmethod
    def from_image(cls, image: bytes) -> ResultMedia:
        # Check if PIL is avaliable
        if PIL is not None:
            image, output = PIL.Image.open(io.BytesIO(image)), io.BytesIO()
            width, height = image.size
            image.save(output, format='PNG')
            return ResultMedia(output.getvalue(), width=width, height=height)

        # Check if OpenCV and NumPy are available
        if cv2 is None or np is None:
            warnings.warn('OpenCV or NumPy not found, image processing disabled')
            return None

        # If image is not a NumPy array, convert it
        if not isinstance(image, np.ndarray):
            image = np.frombuffer(image, dtype=np.uint8)
            image = cv2.imdecode(image, flags=1)

        # Resize the image
        height, width = image.shape[0], image.shape[1]
        image = cv2.resize(image, (round(width / 10), round(height / 10)), interpolation=cv2.INTER_CUBIC)

        # Encode the image to PNG format
        status, buffer = cv2.imencode('.png', image)
        if status:
            return ResultMedia(bytes(buffer), width=width, height=height)

    @classmethod
    def from_video(cls, video: bytes) -> typing.Optional[ResultMedia]:
        # Check if OpenCV is available
        if cv2 is None:
            warnings.warn('OpenCV not found, video processing disabled')
            return None

        # Write video content to a temporary file
        with tempfile.NamedTemporaryFile(mode='wb+', suffix='.mp4') as file:
            file.write(video)

            # Read the video using OpenCV
            capture = cv2.VideoCapture(file.name)
            status, image = capture.read()

            # If successful, calculate video duration and create ResultMedia object
            if status is True:
                fps = capture.get(cv2.CAP_PROP_FPS)
                frames = capture.get(cv2.CAP_PROP_FRAME_COUNT)
                seconds = int(frames / fps) * 1000
                width = image.shape[1]
                height = image.shape[0]

                return ResultMedia(image, width, height, seconds)


from typing import Literal
from pydantic import BaseModel


class ButtonTextbox(BaseModel):
    type_line: Literal['SingleLine', 'MultiLine']
    type_keypad: Literal['String', 'Number']
    place_holder: str
    title: str
    default_value: str


from pydantic import BaseModel
from typing import Optional
from .file import File
from .location import Location
from .aux_data import AuxData


class InlineMessage(BaseModel):
    sender_id: str
    text: str
    file: Optional[File]
    location: Optional[Location]
    aux_data: AuxData
    message_id: str
    chat_id: str


from .on_message_updates import OnMessageUpdates
from .on_remove_notifications import OnRemoveNotifications
from .on_show_notifications import OnShowNotifications
from .on_chat_updates import OnChatUpdates
from .on_show_activities import OnShowActivities


class Decorators(
    OnMessageUpdates,
    OnChatUpdates,
    OnRemoveNotifications,
    OnShowActivities,
    OnShowNotifications,
):
    pass


from typing import Union


class DeleteUserChat:
    async def delete_user_chat(
            self,
            user_guid: str,
            last_deleted_message_id: Union[str, int],
    ):
        return await self.builder('deleteUserChat',
                                  input={
                                      'user_guid': user_guid,
                                      'last_deleted_message_id': last_deleted_message_id,
                                  })


from typing import Optional, Literal
from pydantic import BaseModel
from .aux_data import AuxData
from .file import File
from .forwarded_from import ForwardedFrom
from .location import Location
from .sticker import Sticker
from contact_message import ContactMessage
from .poll import Poll
from .live_location import LiveLocation

class Message(BaseModel):
    message_id: str
    text: Optional[str]
    time: int
    is_edited: bool
    sender_type: Literal['User', 'Bot']
    sender_id: str

    aux_data: Optional[AuxData] = AuxData()
    file: Optional[File]
    reply_to_message_id: Optional[str]
    forwarded_from: Optional[ForwardedFrom]
    forwarded_no_link: Optional[str]
    location: Optional[Location]
    sticker: Optional[Sticker]
    contact_message: Optional[ContactMessage]
    poll: Optional[Poll]
    live_location: Optional[LiveLocation]


import rubpy

from typing import Optional, Union
from pathlib import Path

class SendVideoMessage:
    async def send_video_message(
            self: "rubpy.Client",
            object_guid: str,
            video_message: Union[Path, bytes],
            caption: Optional[str] = None,
            reply_to_message_id: Optional[str] = None,
            auto_delete: Optional[int] = None, *args, **kwargs,
    ):
        """
        Send a video message.

        Args:
            object_guid (str):
                The GUID of the recipient.

            video_message (Path, bytes):
                The photo data.

            caption (str, optional):
                The caption for the video. Defaults to None.

            reply_to_message_id (str, optional):
                The ID of the message to which this is a reply. Defaults to None.
        """

        return await self.send_message(object_guid=object_guid, text=caption, reply_to_message_id=reply_to_message_id, file_inline=video_message, type='VideoMessage', auto_delete=auto_delete, *args, **kwargs)


from typing import Union, Optional
from pathlib import Path
import rubpy

class SendVideo:
    async def send_video(
            self: "rubpy.Client",
            object_guid: str,
            video: Union[Path, bytes],
            caption: Optional[str] = None,
            reply_to_message_id: Optional[str] = None,
            is_spoil: bool = False,
            auto_delete: Optional[int] = None, *args, **kwargs,
    ):
        """
        Send a video.

        Args:
            object_guid (str):
                The GUID of the recipient.

            video (Path, bytes):
                The photo data.

            caption (str, optional):
                The caption for the video. Defaults to None.

            reply_to_message_id (str, optional):
                The ID of the message to which this is a reply. Defaults to None.
        """

        return await self.send_message(
            object_guid=object_guid,
            text=caption,
            reply_to_message_id=reply_to_message_id,
            file_inline=video,
            type='Video',
            is_spoil=is_spoil,
            auto_delete=auto_delete, *args, **kwargs)


import rubpy
from typing import Union

class SeenChannelMessages:
    async def seen_channel_messages(
            self: "rubpy.Client",
            channel_guid: str,
            min_id: Union[int, str],
            max_id: Union[int, str],
    ):
        data = dict(
            channel_guid=channel_guid,
            min_id=min_id,
            max_id=max_id,
        )
        return await self.builder(
            name='seenChannelMessages',
            input=data,
        )


from pydantic import BaseModel
from typing import Optional
from .avatar_thumbnail import AvatarThumbnail
from .chat_reaction_setting import ChatReactionSetting


class Group(BaseModel):
    group_guid: str
    group_title: str
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    count_members: int
    is_deleted: bool
    is_verified: bool
    slow_mode: int
    chat_history_for_new_members: str
    event_messages: bool
    chat_reaction_setting: Optional[ChatReactionSetting] = None


from .results import Results
from .socket_results import SocketResults
from .socket_results import SocketResults as Updates


# from .models import (
#     chats,
#     contacts,
#     extras,
#     groups,
#     messages,
#     stickers,
#     users,
# )


import rubpy


class SignIn:
    async def sign_in(self: "rubpy.Client",
                     phone_code: str,
                     phone_number: str,
                     phone_code_hash: str,
                     public_key: str,
    ):
        return await self.builder(name='signIn',
                                 input={
                                     'phone_code': phone_code,
                                     'phone_number': phone_number,
                                     'phone_code_hash': phone_code_hash,
                                     'public_key': public_key,
                                 },
                                 tmp_session=True,)


from .get_my_gif_set import GetMyGifSet
from .add_to_my_gif_set import AddToMyGifSet
from .remove_from_my_gif_set import RemoveFromMyGifSet


class Gifs(
    GetMyGifSet,
    AddToMyGifSet,
    RemoveFromMyGifSet,
):
    pass


from typing import Optional, List
from pydantic import BaseModel
from .keypad_row import KeypadRow


class Keypad(BaseModel):
    rows: List[KeypadRow]
    resize_keyboard: Optional[bool]
    on_time_keyboard: Optional[bool]


import rubpy

class GetRelatedObjects:
    async def get_related_objects(
            self: "rubpy.Client",
            object_guid: str,
    ):
        return await self.builder(
            name='getRelatedObjects',
            input=dict(object_guid=object_guid),
        )


import rubpy
from typing import Optional, Union


class GetContacts:
    async def get_contacts(
            self: "rubpy.Client",
            start_id: Optional[Union[str, int]] = None,
    ):
        return self.builder(name='getContacts',
                            input={'start_id': str(start_id)})


from pydantic import BaseModel
from typing import Literal, List
from .button_selection_item import ButtonSelectionItem


class ButtonSelection(BaseModel):
    selection_id: str
    search_type: Literal[None, 'Local', 'Api']
    get_type: Literal['Local', 'Api']
    items: List[ButtonSelectionItem]
    is_multi_selection: bool
    columns_count: str
    title: str


from typing import Union
from time import time
import rubpy

class GetFolders:
    async def get_folders(
            self: "rubpy.Client",
            last_state: Union[int, str] = round(time()) - 150,
    ):
        return await self.builder(name='getFolders',
                                  input={'last_state': int(last_state)})


from ... import exceptions
from ...crypto import Crypto
from Crypto.PublicKey import RSA
from Crypto.Signature import pkcs1_15


class Start:
    async def start(self, phone_number: str = None):
        if not hasattr(self, 'connection'):
            await self.connect()

        try:
            #self._logger.info('user info', extra={'data': await self.get_me()})
            self.decode_auth = Crypto.decode_auth(self.auth) if self.auth is not None else None
            self.import_key = pkcs1_15.new(RSA.import_key(self.private_key.encode())) if self.private_key is not None else None
            await self.get_me()

        except exceptions.NotRegistered:
            #self._logger.debug('user not registered!')
            if phone_number is None:
                phone_number = input('Phone Number: ')
                is_phone_number_true = True
                while is_phone_number_true:
                    if input(f'Is the {phone_number} correct[y or n] > ').lower() == 'y':
                        is_phone_number_true = False
                    else:
                        phone_number = input('Phone Number: ')

            if phone_number.startswith('0'):
                phone_number = '98{}'.format(phone_number[1:])
            elif phone_number.startswith('+98'):
                phone_number = phone_number[1:]
            elif phone_number.startswith('0098'):
                phone_number = phone_number[2:]

            result = await self.send_code(phone_number=phone_number)

            if result.status == 'SendPassKey':
                while True:
                    pass_key = input(f'Password [{result.hint_pass_key}] > ')
                    result = await self.send_code(phone_number=phone_number, pass_key=pass_key)

                    if result.status == 'OK':
                        break

            public_key, self.private_key = Crypto.create_keys()
            while True:
                phone_code = input('Code: ')

                result = await self.sign_in(
                    phone_code=phone_code,
                    phone_number=phone_number,
                    phone_code_hash=result.phone_code_hash,
                    public_key=public_key)

                if result.status == 'OK':
                    result.auth = Crypto.decrypt_RSA_OAEP(self.private_key, result.auth)
                    self.key = Crypto.passphrase(result.auth)
                    self.auth = result.auth
                    self.decode_auth = Crypto.decode_auth(self.auth)
                    self.import_key = pkcs1_15.new(RSA.import_key(self.private_key.encode())) if self.private_key is not None else None
                    self.session.insert(
                        auth=self.auth,
                        guid=result.user.user_guid,
                        user_agent=self.user_agent,
                        phone_number=result.user.phone,
                        private_key=self.private_key)

                    await self.register_device()
                    break

        return self


from .delete_avatar import DeleteAvatar
from .delete_chat_history import DeleteChatHistory
from .get_abs_objects import GetAbsObjects
from .get_avatars import GetAvatars
from .get_chats import GetChats
from .get_chats_updates import GetChatsUpdates
from .get_link_from_app_url import GetLinkFromAppUrl
from .upload_avatar import UploadAvatar
from .set_action_chat import SetActionChat
from .send_chat_activity import SendChatActivity
from .seen_chats import SeenChats
from .search_chat_messages import SearchChatMessages


class Chats(
    DeleteAvatar,
    DeleteChatHistory,
    GetAbsObjects,
    GetAvatars,
    GetChats,
    GetChatsUpdates,
    GetLinkFromAppUrl,
    UploadAvatar,
    SetActionChat,
    SendChatActivity,
    SeenChats,
    SearchChatMessages,
):
    pass


from enum import Enum

class ReactionType(int, Enum):
    RED_HEART = 1
    THUMBS_UP, LIKE = 2, 2
    THUMBS_DOWN, UN_LIKE = 3, 3
    FIRE = 4
    SMILING_FACE_WITH_HEARTS = 5
    CLAPPING_HANDS = 6
    IRAN = 7
    BEAMING_FACE = 8
    THINKING_FACE = 9
    EXPLODING_HEAD = 10
    SCREAMING_FACE = 11
    FACE_WITH_SYMBOLS_ON_MOUTH = 12
    CRYING_FACE = 13
    CELEBRATION = 14
    PARTY_POPPER = 15
    STAR_STRUCK = 16
    FACE_VOMITING = 17
    PILE_OF_POO = 18
    FOLDED_HANDS = 19
    OKAY = 20
    DOVE_OF_PEACE = 21
    CLOWN_FACE = 22
    YAWNING_FACE = 23
    WOOZY_FACE = 24
    SMILING_FACE_WITH_HEART_EYES = 25
    WHALE = 26
    HEART_ON_FIRE = 27
    NEW_MOON_FACE = 28
    HOT_DOG = 29
    HUNDRED_POINTS = 30
    ROLLING_ON_THE_FLOOR_LAUGHING = 31
    HIGH_VOLTAGE = 32
    BANANA = 33
    TROPHY = 34
    BROKEN_HEART = 35
    FACE_WITH_RAISED_EYEBROW = 36
    NEUTRAL_FACE = 37
    STRAWBERRY = 38
    KISS_MARK = 39
    SMILING_FACE_WITH_HORNS = 40
    SLEEPING = 41
    LOUDLY_CRYING_FACE = 42
    NERD_FACE = 43
    GHOST = 44
    MAN_TECHNOLOGIST = 45
    EYES = 46
    JACK_O_LANTERN = 47
    SEE_NO_EVIL_MONKEY = 48
    SMILING_FACE_WITH_HALO = 49
    FEARFUL_FACE = 50
    HANDSHAKE = 51
    WRITING_HANDS = 52
    HUGGING_FACE = 53
    SANTA_CLAUS = 54
    CHRISTMAS_TREE = 55
    SNOWMAN = 56
    NAIL_POLISH = 57
    ZANY_FACE = 58
    MOAI = 59
    COOL_BUTTON = 60
    HEART_WITH_ARROW = 61
    HEAR_NO_EVIL_MONKEY = 62
    UNICORN = 63
    FACE_BLOWING_A_KISS = 64
    PILL = 65
    SPEAK_NO_EVIL_MONKEY = 66
    SMILING_FACE_WITH_SUNGLASSES = 67
    ALIEN_MONSTER = 68
    SHRUGGING_MAN = 69
    SHRUGGING_PERSON = 70
    SHRUGGING_WOMAN = 71
    RAGE = 72


import rubpy

class RemoveFromMyGifSet:
    async def remove_from_my_gif_set(
            self: "rubpy.Client",
            file_id: str,
    ):
        return await self.builder('removeFromMyGifSet', input={'file_id': str(file_id)})


from typing import Union
from random import random


class CreatePoll:
    async def create_poll(
            self,
            object_guid: str,
            question: str,
            options: list,
            type: str = 'Regular',
            is_anonymous: bool = True,
            allows_multiple_answers: bool = True,
            correct_option_index: Union[int, str] = None,
            explanation: str = None,
            reply_to_message_id: Union[str, int] = 0,
    ):
        if len(options) <= 1:
            raise ValueError('The `options` argument must have more than two string values.')

        if type not in ('Quiz', 'Regular'):
            raise ValueError('The `type` argument can only be in `["Quiz", "Regular"]`.')

        input = {
            'object_guid': object_guid,
            'question': question,
            'options': options,
            'allows_multiple_answers': allows_multiple_answers,
            'is_anonymous': is_anonymous,
            'reply_to_message_id': reply_to_message_id,
            'type': type,
            'rnd': int(random() * 1e6 + 1),
        }

        if type == 'Quiz':
            input['correct_option_index'] = correct_option_index
            input['explanation'] =  explanation

        return await self.builder('createPoll', input=input)


from pydantic import BaseModel
from typing import Literal


class ButtonCalendar(BaseModel):
    default_value: str
    type: Literal['DatePersian', 'DateGregorian']
    min_year: str
    max_year: str
    title: str


import rubpy
from ... import handlers


class OnShowNotifications:
    def on_show_notifications(
            self: "rubpy.Client",
            *args, **kwargs,
    ):
        def MetaHandler(func):
            self.add_handler(func, handlers.ShowNotifications(*args, **kwargs))
            return func
        return MetaHandler


import sys


class ClientError(Exception):
    pass

class StopHandler(ClientError):
    pass

class APIException(Exception):
    pass

class CancelledError(ClientError):
    pass


class RequestError(ClientError):
    def __init__(self, message, request=None):
        self.message = str(message)
        self.request = request


class UploadError(Exception):
    def __init__(self, status, status_det, dev_message: str=None):
        self.status = status
        self.status_det = status_det
        self.dev_message = dev_message

class CodeIsUsed(RequestError):
    pass


class TooRequests(RequestError):
    pass


class InvalidAuth(RequestError):
    pass


class ServerError(RequestError):
    pass


class UrlNotFound(RequestError):
    pass


class ErrorAction(RequestError):
    pass


class ErrorIgnore(RequestError):
    pass


class ErrorGeneric(RequestError):
    pass


class NoConnection(RequestError):
    pass


class InvalidInput(RequestError):
    pass


class Undeliverable(RequestError):
    pass


class NotRegistered(RequestError):
    pass


class CodeIsExpired(RequestError):
    pass


class InvalidMethod(RequestError):
    pass


class UsernameExist(RequestError):
    pass


class ErrorTryAgain(RequestError):
    pass


class ErrorMessageTry(RequestError):
    pass


class InternalProblem(RequestError):
    pass


class ErrorMessageIgn(RequestError):
    pass


class NotSupportedApiVersion(RequestError):
    pass


class ExcetionsHandler:
    def __init__(self, name) -> None:
        self.name = name

    def __getattr__(self, name):
        name = ''.join([chunk.title() for chunk in name.split('_')])
        return globals().get(name, ClientError)

    def __call__(self, name, *args, **kwargs):
        return getattr(self, name)

sys.modules[__name__] = ExcetionsHandler(__name__)


from ... import exceptions

import rubpy


class Disconnect:
    async def disconnect(self: "rubpy.Client"):
        try:
            return await self.connection.close()
            #self._logger.info(f'the client was disconnected')

        except AttributeError:
            raise exceptions.NoConnection(
                'You must first connect the Client'
                ' with the *.connect() method')


from pydantic import BaseModel, HttpUrl
from typing import List, Optional

class File(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None
    file_name: Optional[str] = None
    cdn_tag: Optional[str] = None

class Sticker(BaseModel):
    emoji_character: Optional[str] = None
    w_h_ratio: Optional[str] = None
    sticker_id: Optional[str] = None
    file: Optional[File] = None
    sticker_set_id: Optional[str] = None

class SetImage(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None
    cdn_tag: Optional[str] = None

class TrendStickerSet(BaseModel):
    sticker_set_id: Optional[str] = None
    title: Optional[str] = None
    count_stickers: Optional[int] = None
    set_image: Optional[SetImage] = None
    top_stickers: Optional[List[Sticker]] = None
    share_string: Optional[HttpUrl] = None
    update_time: Optional[int] = None

class GetTrendStickerSets(BaseModel):
    sticker_sets: Optional[List[TrendStickerSet]] = None
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None

class GetStickersBySetIDs(BaseModel):
    stickers: Optional[List[Sticker]] = []

class StickerSets(BaseModel):
    sticker_set_id: Optional[str] = None
    title: Optional[str] = None
    count_stickers: Optional[int] = None
    set_image: Optional[SetImage] = None
    top_stickers: Optional[List[Sticker]] = []

class GetMyStickerSets(BaseModel):
    sticker_sets: Optional[List[StickerSets]] = []


from datetime import datetime
from pydantic import BaseModel
from .file import File

class Bot(BaseModel):
    bot_id: str
    bot_title: str
    avatar: File
    description: str
    username: str
    start_message: datetime
    share_url: str


import rubpy

class UpdateUsername:
    async def update_username(self: "rubpy.Client", username: str):
        return await self.builder('updateUsername', input={'username': username.replace('@', '')})


from pydantic import BaseModel
from typing import Optional
from .file import File


class Sticker(BaseModel):
    emoji_character: Optional[str] = None
    w_h_ratio: Optional[str] = None
    sticker_id: Optional[str] = None
    file: Optional[File] = None
    sticker_set_id: Optional[str] = None


from .get_object_by_username import GetObjectByUsername
from .search_global_objects import SearchGlobalObjects
from .get_profile_link_items import GetProfileLinkItems
from .ban_member import BanMember
from .get_info import GetInfo
from .join import Join
from .get_related_objects import GetRelatedObjects
from .get_transcription import GetTranscription


class Exctras(
    GetObjectByUsername,
    SearchGlobalObjects,
    GetProfileLinkItems,
    BanMember,
    GetInfo,
    Join,
    GetRelatedObjects,
    GetTranscription,
):
    pass


from json import dumps


class Results:
    def __str__(self) -> str:
        return self.jsonify(indent=2)

    def __getattr__(self, name):
        return self.find_keys(keys=name)

    def __setitem__(self, key, value):
        self.original_update[key] = value

    def __getitem__(self, key):
        return self.original_update[key]

    def __lts__(self, update: list, *args, **kwargs):
        for index, element in enumerate(update):
            if isinstance(element, list):
                update[index] = self.__lts__(update=element)

            elif isinstance(element, dict):
                update[index] = Results(update=element)

            else:
                update[index] = element
        return update

    def __init__(self, update: dict, *args, **kwargs) -> None:
        self.original_update = update

    def to_dict(self):
        return self.original_update

    def jsonify(self, indent=None, *args, **kwargs) -> str:
        result = self.original_update
        result['original_update'] = 'dict{...}'
        return dumps(result, indent=indent,
                          ensure_ascii=False,
                          default=lambda value: str(value))

    def find_keys(self, keys, original_update=None, *args, **kwargs):

        if original_update is None:
            original_update = self.original_update

        if not isinstance(keys, list):
            keys = [keys]

        if isinstance(original_update, dict):
            for key in keys:
                try:
                    update = original_update[key]
                    if isinstance(update, dict):
                        update = Results(update=update)

                    elif isinstance(update, list):
                        update = self.__lts__(update=update)

                    return update

                except KeyError:
                    pass
            original_update = original_update.values()

        for value in original_update:
            if isinstance(value, (dict, list)):
                try:
                    return self.find_keys(keys=keys, original_update=value)

                except AttributeError:
                    return None

        return None


from .. import methods
from .. import thumbnail
from base64 import b64decode
from pydantic import BaseModel
from typing import Optional, List

class FileInline(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None
    file_name: Optional[str] = None
    thumb_inline: Optional[str] = None
    music_performer: Optional[str] = None
    width: Optional[int] = None
    height: Optional[int] = None
    time: Optional[int] = None
    size: Optional[int] = None
    type: Optional[str] = None

class ForwardedFrom(BaseModel):
    type_from: Optional[str] = None
    message_id: Optional[str] = None
    object_guid: Optional[str] = None

class PollOption(BaseModel):
    options: Optional[List[str]] = []
    poll_status: Optional[dict] = None
    is_anonymous: Optional[bool] = None
    type: Optional[str] = None
    allows_multiple_answers: Optional[bool] = None

class Poll(BaseModel):
    poll_id: Optional[str] = None
    question: Optional[str] = None
    options: Optional[List[str]] = None
    poll_status: Optional[dict] = None
    is_anonymous: Optional[bool] = None
    type: Optional[str] = None
    allows_multiple_answers: Optional[bool] = None

class AvatarThumbnail(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None

class MessageData(BaseModel):
    message_id: Optional[str] = None
    text: Optional[str] = None
    file_inline: Optional[FileInline] = None
    time: Optional[str] = None
    is_edited: Optional[bool] = None
    type: Optional[str] = None
    author_type: Optional[str] = None
    author_object_guid: Optional[str] = None
    forwarded_from: Optional[ForwardedFrom] = None
    poll: Optional[Poll] = None

class AbsObject(BaseModel):
    object_guid: Optional[str] = None
    type: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None

class Message(BaseModel):
    message_id: Optional[str] = None
    text: Optional[str] = None
    file_inline: Optional[FileInline] = None
    time: Optional[str] = None
    is_edited: Optional[bool] = None
    type: Optional[str] = None
    author_type: Optional[str] = None
    author_object_guid: Optional[str] = None
    forwarded_from: Optional[ForwardedFrom] = None
    poll: Optional[Poll] = None
    object_guid: Optional[str] = None
    message: Optional[MessageData] = None
    abs_object: Optional[AbsObject] = None
    last_seen_peer_mid: Optional[int] = None

class MessageUpdate(BaseModel):
    message_id: Optional[str] = None
    action: Optional[str] = None
    message: Optional[Message] = None
    updated_parameters: Optional[List] = None
    timestamp: Optional[str] = None
    prev_message_id: Optional[str] = None
    object_guid: Optional[str] = None
    type: Optional[str] = None
    state: Optional[str] = None

class LastMessage(BaseModel):
    message_id: Optional[str] = None
    type: Optional[str] = None
    text: Optional[str] = None
    author_object_guid: Optional[str] = None
    is_mine: Optional[bool] = None
    author_title: Optional[str] = None
    author_type: Optional[str] = None

class Chat(BaseModel):
    time_string: Optional[str] = None
    last_message: Optional[LastMessage] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[int] = None
    status: Optional[str] = None
    time: Optional[int] = None
    last_message_id: Optional[str] = None

class ChatUpdate(BaseModel):
    object_guid: Optional[str] = None
    action: Optional[str] = None
    chat: Optional[Chat] = None
    updated_parameters: Optional[List] = None
    timestamp: Optional[str] = None
    type: Optional[str] = None

class SendMessage(BaseModel):
    message_update: Optional[MessageUpdate] = None
    status: Optional[str] = None
    chat_update: Optional[ChatUpdate] = None

    @classmethod
    async def set_shared_data(cls, client, message):
        cls._client = client
        cls._message = message

    @classmethod
    async def pin(cls,
                  object_guid: str = None,
                  message_id: str = None,
                  action: str = methods.messages.Pin, *args, **kwargs):
        """_pin_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_id (str, optional):
                _custom message id_. Defaults to update.message_id.
            action (bool, optional):
                _pin or unpin_. Defaults to methods.messages.Pin. (
                    methods.messages.Pin,
                    methods.messages.Unpin
                )
        Returns:
            BaseResults: result
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_id is None:
            message_id = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.SetPinMessage(
                object_guid=object_guid,
                message_id=message_id,
                action=action))

    @classmethod
    async def edit(cls,
                   text: str,
                   object_guid: str = None,
                   message_id: str = None, *args, **kwargs):
        """_edit_
        Args:
            text (str):
                _message text_
            object_guid (str, optional):
                _custom objec guid_. Defaults to update.object_guid.
            message_id (str, optional):
                _custom message id_. Defaults to update.message_id.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_id is None:
            message_id = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.EditMessage(
                object_guid=object_guid,
                message_id=message_id,
                text=text))
    
    @classmethod
    async def copy(cls,
                   to_object_guid: str,
                   from_object_guid: str = None, message_ids=None, *args, **kwargs):
        """_copy_
        Args:
            to_object_guid (str):
                _to object guid_.
            from_object_guid (str, optional):
                _from object guid_. Defaults to update.object_guid.
            message_ids (typing.Union[str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """
            
        if from_object_guid is None:
            from_object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id
        
        result = await cls.get_messages(from_object_guid, message_ids)
        messages = []
        if result.messages:
            for message in result.messages:
                
                try:
                    file_inline = message.file_inline
                    kwargs.update(file_inline.to_dict())

                except AttributeError:
                    file_inline = None

                try:
                    thumb = thumbnail.Thumbnail(
                        b64decode(message.thumb_inline), *args, **kwargs)
                    
                except AttributeError:
                    thumb = kwargs.get('thumb', True)
                                
                try:
                    message = message.sticker
                
                except AttributeError:
                    message = message.raw_text
                
                if file_inline is not None:
                    if file_inline.type not in [methods.messages.Gif,
                                                methods.messages.Sticker]:
                        file_inline = await cls.download(file_inline)
                        messages.append(await cls._client.send_message(
                            thumb=thumb,
                            message=message,
                            file_inline=file_inline,
                            object_guid=to_object_guid, *args, **kwargs))
                        continue

                messages.append(await cls._client.send_message(
                    message=message,
                    object_guid=to_object_guid,
                    file_inline=file_inline, *args, **kwargs))
    
        return {'status': 'OK', 'messages': messages}

    @classmethod
    async def seen(cls, seen_list: dict = None, *args, **kwargs):
        """_seen_
        Args:
            seen_list (dict, optional):
                _description_. Defaults t
                    {update.object_guid: update.message_id}
        """

        if seen_list is None:
            seen_list = {cls._message.message_update.object_guid: cls._message.message_update.message_id}
        return await cls._client(methods.chats.SeenChats(seen_list=seen_list))

    @classmethod
    async def reply(cls,
                    message=None,
                    object_guid: str = None,
                    reply_to_message_id: str = None,
                    file_inline: str = None, *args, **kwargs):
        """_reply_
        Args:
            message (Any, optional):
                _message or cation or sticker_ . Defaults to None.
            object_guid (str):
                _object guid_ . Defaults to update.object_guid
            reply_to_message_id (str, optional):
                _reply to message id_. Defaults to None.
            file_inline (typing.Union[pathlib.Path, bytes], optional):
                _file_. Defaults to None.
            is_gif (bool, optional):
                _is it a gif file or not_. Defaults to None.
            is_image (bool, optional):
                _is it a image file or not_. Defaults to None.
            is_voice (bool, optional):
                _is it a voice file or not_. Defaults to None.
            is_music (bool, optional):
                _is it a music file or not_. Defaults to None.
            is_video (bool, optional):
                _is it a video file or not_. Defaults to None.
            thumb (bool, optional):
                if value is "True",
                the lib will try to build the thumb ( require cv2 )
                if value is thumbnail.Thumbnail, to set custom
                Defaults to True.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_id is None:
            message_id = cls._message.message_update.message_id

        return await cls._client.send_message(
            message=message,
            object_guid=object_guid,
            file_inline=file_inline,
            reply_to_message_id=reply_to_message_id, *args, **kwargs)

    @classmethod
    async def forwards(cls,
                       to_object_guid: str,
                       from_object_guid: str = None,
                       message_ids=None, *args, **kwargs):
        """_forwards_
        Args:
            to_object_guid (str):
                _to object guid_.
            from_object_guid (str, optional):
                _from object guid_. Defaults to update.object_guid.
            message_ids (typing.Union[str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """

        if from_object_guid is None:
            from_object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.ForwardMessages(
                from_object_guid=from_object_guid,
                to_object_guid=to_object_guid,
                message_ids=message_ids))

    @classmethod
    async def download(cls, file_inline=None, file=None, *args, **kwargs):
        return await cls._client.download_file_inline(
            file_inline or cls._message.file_inline,
            file=file, *args, **kwargs)

    @classmethod
    async def get_author(cls, author_guid: str = None, *args, **kwargs):
        """_get user or author information_
        Args:
            author_guid (str, optional):
                _custom author guid_. Defaults to update.author_guid
        """

        if author_guid is None:
            author_guid = cls._message.message_update.message.author_object_guid

        return await cls.get_object(object_guid=author_guid, *args, **kwargs)

    @classmethod
    async def get_object(cls, object_guid: str = None, *args, **kwargs):
        """_get object information_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid


        if cls.guid_type(object_guid) == 'User':
            return await cls._client(
                methods.users.GetUserInfo(
                    user_guid=object_guid))

        elif cls.guid_type(object_guid) == 'Group':
            return await cls._client(
                methods.groups.GetGroupInfo(
                    object_guid=object_guid))

        elif cls.guid_type(object_guid) == 'Channel':
            return await cls._client(
                methods.channels.GetChannelInfo(
                    object_guid=object_guid))

    @classmethod
    async def get_message(cls,
                           object_guid: str = None,
                           message_ids=None, *args, **kwargs):
        """_get messages_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_ids (str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.GetMessagesByID(
                object_guid=object_guid, message_ids=message_ids))

    @classmethod
    async def delete(cls,
                              object_guid: str = None,
                              message_ids=None, *args, **kwargs):
        """_delete messages_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_ids (str, list, optional):
                _custom message ids_. Defaults to update.message_id.
            type(str, optional):
                the message should be deleted for everyone or not.
                Defaults to methods.messages.Global (
                    methods.messages.Local,
                    methods.messages.Global
                )
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.DeleteMessages(
                object_guid=object_guid,
                message_ids=message_ids, *args, **kwargs))

class EditMessage(BaseModel):
    message_update: Optional[MessageUpdate] = None
    chat_update: Optional[ChatUpdate] = None
    
    @classmethod
    async def set_shared_data(cls, client, message):
        cls._client = client
        cls._message = message

    @classmethod
    async def pin(cls,
                  object_guid: str = None,
                  message_id: str = None,
                  action: str = methods.messages.Pin, *args, **kwargs):
        """_pin_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_id (str, optional):
                _custom message id_. Defaults to update.message_id.
            action (bool, optional):
                _pin or unpin_. Defaults to methods.messages.Pin. (
                    methods.messages.Pin,
                    methods.messages.Unpin
                )
        Returns:
            BaseResults: result
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_id is None:
            message_id = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.SetPinMessage(
                object_guid=object_guid,
                message_id=message_id,
                action=action))

    @classmethod
    async def edit(cls,
                   text: str,
                   object_guid: str = None,
                   message_id: str = None, *args, **kwargs):
        """_edit_
        Args:
            text (str):
                _message text_
            object_guid (str, optional):
                _custom objec guid_. Defaults to update.object_guid.
            message_id (str, optional):
                _custom message id_. Defaults to update.message_id.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_id is None:
            message_id = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.EditMessage(
                object_guid=object_guid,
                message_id=message_id,
                text=text))
    
    @classmethod
    async def copy(cls,
                   to_object_guid: str,
                   from_object_guid: str = None, message_ids=None, *args, **kwargs):
        """_copy_
        Args:
            to_object_guid (str):
                _to object guid_.
            from_object_guid (str, optional):
                _from object guid_. Defaults to update.object_guid.
            message_ids (typing.Union[str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """
            
        if from_object_guid is None:
            from_object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id
        
        result = await cls.get_messages(from_object_guid, message_ids)
        messages = []
        if result.messages:
            for message in result.messages:
                
                try:
                    file_inline = message.file_inline
                    kwargs.update(file_inline.to_dict())

                except AttributeError:
                    file_inline = None

                try:
                    thumb = thumbnail.Thumbnail(
                        b64decode(message.thumb_inline), *args, **kwargs)
                    
                except AttributeError:
                    thumb = kwargs.get('thumb', True)
                                
                try:
                    message = message.sticker
                
                except AttributeError:
                    message = message.raw_text
                
                if file_inline is not None:
                    if file_inline.type not in [methods.messages.Gif,
                                                methods.messages.Sticker]:
                        file_inline = await cls.download(file_inline)
                        messages.append(await cls._client.send_message(
                            thumb=thumb,
                            message=message,
                            file_inline=file_inline,
                            object_guid=to_object_guid, *args, **kwargs))
                        continue

                messages.append(await cls._client.send_message(
                    message=message,
                    object_guid=to_object_guid,
                    file_inline=file_inline, *args, **kwargs))
    
        return {'status': 'OK', 'messages': messages}

    @classmethod
    async def seen(cls, seen_list: dict = None, *args, **kwargs):
        """_seen_
        Args:
            seen_list (dict, optional):
                _description_. Defaults t
                    {update.object_guid: update.message_id}
        """

        if seen_list is None:
            seen_list = {cls._message.message_update.object_guid: cls._message.message_update.message_id}
        return await cls._client(methods.chats.SeenChats(seen_list=seen_list))

    @classmethod
    async def reply(cls,
                    message=None,
                    object_guid: str = None,
                    reply_to_message_id: str = None,
                    file_inline: str = None, *args, **kwargs):
        """_reply_
        Args:
            message (Any, optional):
                _message or cation or sticker_ . Defaults to None.
            object_guid (str):
                _object guid_ . Defaults to update.object_guid
            reply_to_message_id (str, optional):
                _reply to message id_. Defaults to None.
            file_inline (typing.Union[pathlib.Path, bytes], optional):
                _file_. Defaults to None.
            is_gif (bool, optional):
                _is it a gif file or not_. Defaults to None.
            is_image (bool, optional):
                _is it a image file or not_. Defaults to None.
            is_voice (bool, optional):
                _is it a voice file or not_. Defaults to None.
            is_music (bool, optional):
                _is it a music file or not_. Defaults to None.
            is_video (bool, optional):
                _is it a video file or not_. Defaults to None.
            thumb (bool, optional):
                if value is "True",
                the lib will try to build the thumb ( require cv2 )
                if value is thumbnail.Thumbnail, to set custom
                Defaults to True.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_id is None:
            message_id = cls._message.message_update.message_id

        return await cls._client.send_message(
            message=message,
            object_guid=object_guid,
            file_inline=file_inline,
            reply_to_message_id=reply_to_message_id, *args, **kwargs)

    @classmethod
    async def forwards(cls,
                       to_object_guid: str,
                       from_object_guid: str = None,
                       message_ids=None, *args, **kwargs):
        """_forwards_
        Args:
            to_object_guid (str):
                _to object guid_.
            from_object_guid (str, optional):
                _from object guid_. Defaults to update.object_guid.
            message_ids (typing.Union[str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """

        if from_object_guid is None:
            from_object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.ForwardMessages(
                from_object_guid=from_object_guid,
                to_object_guid=to_object_guid,
                message_ids=message_ids))

    @classmethod
    async def download(cls, file_inline=None, file=None, *args, **kwargs):
        return await cls._client.download_file_inline(
            file_inline or cls._message.file_inline,
            file=file, *args, **kwargs)

    @classmethod
    async def get_author(cls, author_guid: str = None, *args, **kwargs):
        """_get user or author information_
        Args:
            author_guid (str, optional):
                _custom author guid_. Defaults to update.author_guid
        """

        if author_guid is None:
            author_guid = cls._message.message_update.message.author_object_guid

        return await cls.get_object(object_guid=author_guid, *args, **kwargs)

    @classmethod
    async def get_object(cls, object_guid: str = None, *args, **kwargs):
        """_get object information_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid


        if cls.guid_type(object_guid) == 'User':
            return await cls._client(
                methods.users.GetUserInfo(
                    user_guid=object_guid))

        elif cls.guid_type(object_guid) == 'Group':
            return await cls._client(
                methods.groups.GetGroupInfo(
                    object_guid=object_guid))

        elif cls.guid_type(object_guid) == 'Channel':
            return await cls._client(
                methods.channels.GetChannelInfo(
                    object_guid=object_guid))

    @classmethod
    async def get_message(cls,
                           object_guid: str = None,
                           message_ids=None, *args, **kwargs):
        """_get messages_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_ids (str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.GetMessagesByID(
                object_guid=object_guid, message_ids=message_ids))

    @classmethod
    async def delete(cls,
                              object_guid: str = None,
                              message_ids=None, *args, **kwargs):
        """_delete messages_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_ids (str, list, optional):
                _custom message ids_. Defaults to update.message_id.
            type(str, optional):
                the message should be deleted for everyone or not.
                Defaults to methods.messages.Global (
                    methods.messages.Local,
                    methods.messages.Global
                )
        """

        if object_guid is None:
            object_guid = cls._message.message_update.object_guid

        if message_ids is None:
            message_ids = cls._message.message_update.message_id

        return await cls._client(
            methods.messages.DeleteMessages(
                object_guid=object_guid,
                message_ids=message_ids, *args, **kwargs))

class DeleteMessages(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    message_updates: Optional[List[MessageUpdate]] = []
    _client: Optional[str] = None
    original_update: Optional[str] = None

class RequestSendFile(BaseModel):
    id: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_send: Optional[str] = None
    upload_url: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class ForwardMessages(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    message_updates: Optional[List[MessageUpdate]] = []
    status: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class PollStatus(BaseModel):
    state: Optional[str] = None
    selection_index: Optional[int] = None
    percent_vote_options: Optional[List[int]] = None
    total_vote: Optional[int] = None
    show_total_votes: Optional[bool] = None

class VotePoll(BaseModel):
    poll_status: Optional[PollStatus] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class SetPinMessage(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    status: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetMessagesUpdates(BaseModel):
    updated_messages: Optional[List[MessageUpdate]] = []
    new_state: Optional[int] = None
    status: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class SearchGlobalMessages(BaseModel):
    messages: Optional[List[Message]] = []
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetMessagesByID(BaseModel):
    messages: Optional[List[Message]] = []
    timestamp: Optional[int] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetMessagesInterval(BaseModel):
    messages: Optional[List[Message]] = []
    state: Optional[str] = None
    new_has_continue: Optional[bool] = None
    old_has_continue: Optional[bool] = None
    new_min_id: Optional[int] = None
    old_max_id: Optional[int] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class Reactions(BaseModel):
    user_guids: Optional[List] = []
    reaction_count: Optional[int] = None
    emoji_char: Optional[str] = None
    reaction_id: Optional[int] = None
    is_selected: Optional[bool] = None

class Reacion(BaseModel):
    reactions: Optional[List[Reactions]] = []


from .add_channel import AddChannel
from .add_channel_members import AddChannelMembers
from .ban_channel_member import BanChannelMember
from .channel_preview_by_join_link import ChannelPreviewByJoinLink
from .check_channel_username import CheckChannelUsername
from .create_channel_voice_chat import CreateChannelVoiceChat
from .delete_no_access_group_chat import DeleteNoAccessGroupChat
from .discard_channel_voice_chat import DiscardChannelVoiceChat
from .edit_channel_info import EditChannelInfo
from .get_banned_group_members import GetBannedGroupMembers
from .get_channel_admin_access_list import GetChannelAdminAccessList
from .get_channel_admin_members import GetChannelAdminMembers
from .get_channel_all_members import GetChannelAllMembers
from .get_channel_info import GetChannelInfo
from .get_channel_link import GetChannelLink
from .get_group_default_access import GetGroupDefaultAccess
from .get_group_mention_list import GetGroupMentionList
from .get_group_voice_chat_updates import GetGroupVoiceChatUpdates
from .join_channel_action import JoinChannelAction
from .join_channel_by_link import JoinChannelByLink
from .join_group import JoinGroup
from .leave_group import LeaveGroup
from .leave_group_voice_chat import LeaveGroupVoiceChat
from .remove_channel import RemoveChannel
from .set_channel_link import SetChannelLink
from .set_channel_voice_chat_setting import SetChannelVoiceChatSetting
from .set_group_admin import SetGroupAdmin
from .set_group_default_access import SetGroupDefaultAccess
from .update_channel_username import UpdateChannelUsername
from .seen_channel_messages import SeenChannelMessages


class Channels(
        AddChannel,
        AddChannelMembers,
        BanChannelMember,
        ChannelPreviewByJoinLink,
        CheckChannelUsername,
        CreateChannelVoiceChat,
        DeleteNoAccessGroupChat,
        DiscardChannelVoiceChat,
        EditChannelInfo,
        GetBannedGroupMembers,
        GetChannelAdminAccessList,
        GetChannelAdminMembers,
        GetChannelAllMembers,
        GetChannelInfo,
        GetChannelLink,
        GetGroupDefaultAccess,
        GetGroupMentionList,
        GetGroupVoiceChatUpdates,
        JoinChannelAction,
        JoinChannelByLink,
        JoinGroup,
        LeaveGroup,
        LeaveGroupVoiceChat,
        RemoveChannel,
        SetChannelLink,
        SetChannelVoiceChatSetting,
        SetGroupAdmin,
        SetGroupDefaultAccess,
        UpdateChannelUsername,
        SeenChannelMessages,
):
        pass


from typing import List, Optional
from pydantic import BaseModel
from .member import Member


class InChatMembers(BaseModel):
    in_chat_members: Optional[List[Member]]
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None
    timestamp: Optional[str] = None


from typing import Literal, List, Optional
from .types import *

from .. import exceptions
import aiorubika

class Bot:
    BASE_URL = 'https://messengerg2b1.iranlms.ir/v3/'

    def __init__(self, client: "aiorubika.Client") -> None:
        self.client = client
        self.url = f'{self.BASE_URL}{client.bot_token}/'

    async def execute(self, method: str, data: dict):
        result = await self.client.connection.send(
            api_version='bot',
            url=self.url + method,
            input=data,
        )

        if result.get('status') != 'OK':
            raise exceptions.APIException(result)

        return result.get('data')

    async def _send_message(self, method: str, data: dict) -> str:
        res = await self.execute(method=method, data=data)
        return res['message_id']

    async def get_me(self) -> Bot:
        bot = await self.execute(method='getMe', data={})
        return Bot(**bot)

    async def send_message(
        self,
        chat_id: str,
        text: str,
        chat_keypad: Optional[Keypad] = None,
        disable_notification: bool = False,
        inline_keypad: Optional[Keypad] = None,
        reply_to_message_id: Optional[str] = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
    ) -> str:
        data = {
            'text': text,
            'chat_id': chat_id,
            'disable_notification': disable_notification
        }
        if chat_keypad:
            data['chat_keypad'] = chat_keypad.dict()
            data['chat_keypad_type'] = chat_keypad_type
        if inline_keypad:
            data['inline_keypad'] = inline_keypad.dict()
        if reply_to_message_id:
            data['reply_to_message_id'] = reply_to_message_id

        return await self._send_message(method='sendMessage', data=data)

    async def send_poll(
        self,
        chat_id: str,
        question: str,
        options: List[str],
        chat_keypad: Optional[Keypad] = None,
        disable_notification: bool = False,
        inline_keypad: Optional[Keypad] = None,
        reply_to_message_id: Optional[str] = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
    ) -> str:
        data = {
            'chat_id': chat_id,
            'options': options,
            'question': question,
            'chat_keypad': chat_keypad,
            'inline_keypad': inline_keypad,
            'chat_keypad_type': chat_keypad_type,
            'reply_to_message_id': reply_to_message_id,
            'disable_notification': disable_notification,
        }

        return await self._send_message(method='sendPoll', data=data)

    async def send_location(
        self,
        chat_id: str,
        latitude: str,
        longitude: str,
        chat_keypad: Optional[Keypad] = None,
        disable_notification: bool = False,
        inline_keypad: Optional[Keypad] = None,
        reply_to_message_id: Optional[str] = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
    ) -> str:
        data = {
            'chat_id': chat_id,
            'latitude': latitude,
            'longitude': longitude,
            'chat_keypad': chat_keypad,
            'disable_notification': disable_notification,
            'inline_keypad': inline_keypad,
            'reply_to_message_id': reply_to_message_id,
            'chat_keypad_type': chat_keypad_type,
        }

        return await self._send_message(method='sendLocation', data=data)

    async def send_sticker(
        self,
        chat_id: str,
        sticker_id: str,
        chat_keypad: Optional[Keypad] = None,
        disable_notification: bool = False,
        inline_keypad: Optional[Keypad] = None,
        reply_to_message_id: Optional[str] = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
    ) -> str:
        data = {
            'chat_id': chat_id,
            'sticker_id': sticker_id,
            'chat_keypad': chat_keypad,
            'disable_notification': disable_notification,
            'inline_keypad': inline_keypad,
            'reply_to_message_id': reply_to_message_id,
            'chat_keypad_type': chat_keypad_type,
        }

        return await self._send_message(method='sendSticker', data=data)

    async def send_contact(
        self,
        chat_id: str,
        first_name: str,
        last_name: str,
        phone_number: str,
        chat_keypad: Optional[Keypad] = None,
        disable_notification: bool = False,
        inline_keypad: Optional[Keypad] = None,
        reply_to_message_id: Optional[str] = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
    ) -> str:
        data = {
            'chat_id': chat_id,
            'last_name': last_name,
            'first_name': first_name,
            'chat_keypad': chat_keypad,
            'phone_number': phone_number,
            'inline_keypad': inline_keypad,
            'chat_keypad_type': chat_keypad_type,
            'reply_to_message_id': reply_to_message_id,
            'disable_notification': disable_notification,
        }

        return await self._send_message(method='sendContact', data=data)

    async def upload(self, url: str, file_name: str, file_path: str) -> str:
        res = await self._send_multipart_request(url=url, form_data={'file': (file_name, open(file_path, 'rb'))})
        return res['file_id']

    async def get_file(self, file_id: str) -> str:
        data = {'file_id': file_id}
        res = await self.execute(method='getFile', data=data)
        return res['download_url']

    async def set_commands(self, bot_commands: List[BotCommand]) -> None:
        data = {'bot_commands': [command.dict() for command in bot_commands]}
        await self.execute(method='setCommands', data=data)

    async def edit_chat_keypad(self, chat_id: str, chat_keypad: Keypad) -> None:
        data = {
            'chat_id': chat_id,
            'chat_keypad_type': 'New',
            'chat_keypad': chat_keypad.dict(),
        }
        await self.execute(method='editChatKeypad', data=data)

    async def remove_chat_keypad(self, chat_id: str) -> None:
        data = {
            'chat_id': chat_id,
            'chat_keypad_type': 'Remove',
        }
        await self.execute(method='editChatKeypad', data=data)

    async def update_bot_endpoint(
            self,
            token: str,
            url: str,
            type: Literal['ReceiveUpdate', 'ReceiveInlineMessage', 'ReceiveQuery',
                        'GetSelectionItem', 'SearchSelectionItems'],
    ):# -> Final[Union['Done', 'InvalidUrl']]:
        data = {
            'url': url,
            'type': type,
        }
        res = await self.execute(method='updateBotEndpoints', data=data)
        return res['status']

        # ... (previous code)

    async def forward_message(
            self,
            from_chat_id: str,
            message_id: str,
            to_chat_id: str,
            disable_notification: bool = False
    ) -> str:
        data = {
            'from_chat_id': from_chat_id,
            'message_id': message_id,
            'to_chat_id': to_chat_id,
            'disable_notification': disable_notification,
        }
        res = await self.execute(method='forwardMessage', data=data)
        return res['new_message_id']

    async def edit_message_text(self, chat_id: str, message_id: str, text: str) -> None:
        data = {
            'text': text,
            'chat_id': chat_id,
            'message_id': message_id,
        }
        await self.execute(method='editMessageText', data=data)

    async def edit_message_keypad(self, chat_id: str, message_id: str, inline_keypad: Keypad) -> None:
        data = {
            'chat_id': chat_id,
            'message_id': message_id,
            'inline_keypad': inline_keypad.dict(),
        }
        await self.execute(method='editMessageKeypad', data=data)

    async def delete_message(self, chat_id: str, message_id: str) -> None:
        data = {
            'chat_id': chat_id,
            'message_id': message_id,
        }
        await self.execute(method='deleteMessage', data=data)

    async def send_file(
            self,
            token: str,
            chat_id: str,
            file_id: str,
            chat_keypad: Optional[Keypad] = None,
            disable_notification: bool = False,
            inline_keypad: Optional[Keypad] = None,
            reply_to_message_id: Optional[str] = None,
            chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
    ) -> str:
        data = {
            'chat_id': chat_id,
            'file_id': file_id,
            'chat_keypad': chat_keypad,
            'disable_notification': disable_notification,
            'inline_keypad': inline_keypad,
            'reply_to_message_id': reply_to_message_id,
            'chat_keypad_type': chat_keypad_type,

        }
        res = await self.execute(method='sendFile', data=data)
        return res['message_id']

    async def request_send_file(self, type: Literal['File', 'Image', 'Voice', 'Music', 'Gif']) -> str:
        data = {
            'type': type,
        }
        res = await self.execute(method='requestSendFile', data=data)
        return res['upload_url']

    async def upload(self, url: str, file_name: str, file_path: str) -> str:
        res = await self._send_multipart_request(url=url, form_data={'file': (file_name, open(file_path, 'rb'))})
        return res['file_id']

    async def get_file(self, file_id: str) -> str:
        data = {'file_id': file_id}
        res = await self.execute(method='getFile', data=data)
        return res['download_url']

    async def set_commands(self, bot_commands: List[BotCommand]) -> None:
        data = {'bot_commands': [command.dict() for command in bot_commands]}
        await self.execute(method='setCommands', data=data)

    async def edit_chat_keypad(self, chat_id: str, chat_keypad: Keypad) -> None:
        data = {
            'chat_id': chat_id,
            'chat_keypad_type': 'New',
            'chat_keypad': chat_keypad.dict(),
        }
        await self.execute(method='editChatKeypad', data=data)

    async def remove_chat_keypad(self, chat_id: str) -> None:
        data = {
            'chat_id': chat_id,
            'chat_keypad_type': 'Remove',
        }
        await self.execute(method='editChatKeypad', data=data)

    async def update_bot_endpoint(
            self,
            url: str,
            type: Literal['ReceiveUpdate', 'ReceiveInlineMessage', 'ReceiveQuery',
                        'GetSelectionItem', 'SearchSelectionItems'],
    ): # -> Final[Union['Done', 'InvalidUrl']]:
        data = {
            'url': url,
            'type': type,
        }
        res = await self.execute(method='updateBotEndpoints', data=data)
        return res['status']

# ... (remaining code)



from typing import Union


class ForwardMessages:
    async def forward_messages(
            self,
            from_object_guid: str,
            to_object_guid: str,
            message_ids: Union[str, int, list],
    ):
        if not isinstance(message_ids, list):
            message_ids = [str(message_ids)]

        return await self.builder('forwardMessages',
                                  input={
                                      'from_object_guid': from_object_guid,
                                      'to_object_guid': to_object_guid,
                                      'message_ids': message_ids,
                                  })


import rubpy

class AddAddressBook:
    async def add_address_book(
            self: "rubpy.Client",
            phone: str,
            first_name: str,
            last_name: str = '',
    ):
        input = {
            'phone': phone,
            'first_name': str(first_name),
            'last_name': str(last_name),
        }

        return await self.builder(name='addAddressBook',
                                  input=input)


import rubpy
import asyncio

class AutoDeleteMessage:
    async def auto_delete_message(
            self: "rubpy.Client",
            object_guid: str,
            message_id: str,
            time: int,
    ):
        await asyncio.sleep(time)
        return await self.delete_messages(object_guid, message_id)


from random import random
import os
from typing import Dict, Union

from .. import exceptions
from ..types import Results
import rubpy

class Rubino:
    DEFAULT_PLATFORM = {
        'app_name': 'Main',
        'app_version': '4.4.6',
        'platform': 'PWA',
        'package': 'web.rubika.ir',
    }

    def __init__(self, client: 'rubpy.Client') -> None:
        self.client = client
        self.url = 'https://rubino1.iranlms.ir/'

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args, **kwargs):
        return

    async def _execute_request(self, method: str, data: Dict[str, Union[int, str, bool]]) -> Results:
        result = await self.client.connection.send(
            api_version='0',
            input=data,
            method=method,
            url=self.url,
            client=self.DEFAULT_PLATFORM,
        )

        if result.get('status') == 'OK':
            return Results(result.get('data'))
        else:
            raise exceptions.ErrorAction(result.get('status_det'))

    async def _follow_unfollow_helper(self, f_type: str, followee_id: str, profile_id: str) -> dict:
        return await self._execute_request(
            'requestFollow',
            {
                'f_type': f_type,
                'followee_id': followee_id,
                'profile_id': profile_id
            })

    async def get_profile_list(self, limit: int = 10, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getProfileList', {'limit': limit, 'sort': sort, 'equal': equal})

    async def follow(self, followee_id: str, profile_id: str) -> dict:
        return await self._follow_unfollow_helper('Follow', followee_id, profile_id)

    async def unfollow(self, followee_id: str, profile_id: str) -> dict:
        return await self._follow_unfollow_helper('Unfollow', followee_id, profile_id)

    async def get_my_profile_info(self, profile_id: str = None) -> dict:
        return await self._execute_request('getMyProfileInfo', {'profile_id': profile_id})

    async def create_page(self, **kwargs) -> dict:
        return await self._execute_request('createPage', {**kwargs})

    async def update_profile(self, **kwargs) -> dict:
        return await self._execute_request('updateProfile', {**kwargs})

    async def is_exist_username(self, username: str) -> dict:
        return await self._execute_request('isExistUsername', {'username': username.replace('@', '')})

    async def get_post_by_share_link(self, share_link: str, profile_id: str = None) -> dict:
        return await self._execute_request('getPostByShareLink', {'share_string': share_link.split('/')[-1], 'profile_id': profile_id})

    async def add_comment(self, text: str, post_id: str, post_profile_id: str, profile_id: str) -> dict:
        return await self._execute_request('addComment', {'content': text, 'post_id': post_id,
                                                   'post_profile_id': post_profile_id,
                                                   'rnd': int(random() * 1e10),
                                                   'profile_id': profile_id})

    async def _like_unlike_helper(self, action_type: str, post_id: str, post_profile_id: str, profile_id: str) -> dict:
        return await self._execute_request('likePostAction', {'action_type': action_type,
                                                      'post_id': post_id,
                                                      'post_profile_id': post_profile_id,
                                                      'profile_id': profile_id})

    async def like(self, post_id: str, post_profile_id: str, profile_id: str) -> dict:
        return await self._like_unlike_helper('Like', post_id, post_profile_id, profile_id)

    async def unlike(self, post_id: str, post_profile_id: str, profile_id: str) -> dict:
        return await self._like_unlike_helper('Unlike', post_id, post_profile_id, profile_id)

    async def view(self, post_id: str, post_profile_id: str) -> dict:
        return await self._execute_request('addPostViewCount', {'post_id': post_id, 'post_profile_id': post_profile_id})

    # ... (previous code)

    async def get_comments(self, post_id: str, profile_id: str, post_profile_id: str,
                           limit: int = 50, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getComments', {
            'equal': equal,
            'limit': limit,
            'sort': sort,
            'post_id': post_id,
            'profile_id': profile_id,
            'post_profile_id': post_profile_id
        })

    async def get_recent_following_posts(self, profile_id: str, limit: int = 30,
                                         sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getRecentFollowingPosts', {
            'equal': equal,
            'limit': limit,
            'sort': sort,
            'profile_id': profile_id
        })

    async def get_profile_posts(self, target_profile_id: str, profile_id: str,
                                limit: int = 50, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getRecentFollowingPosts', {
            'equal': equal,
            'limit': limit,
            'sort': sort,
            'profile_id': profile_id,
            'target_profile_id': target_profile_id
        })

    async def get_profiles_stories(self, profile_id: str, limit: int = 100) -> dict:
        return await self._execute_request('getProfilesStories', {
            'limit': limit,
            'profile_id': profile_id
        })

    async def request_upload_file(self, profile_id: str, file_name: str,
                                  file_size: int, file_type: str) -> dict:
        return await self._execute_request('requestUploadFile', {
            'file_name': file_name.split('/')[-1],
            'file_size': file_size,
            'file_type': file_type,
            'profile_id': profile_id
        })

    async def get_profile_highlights(self, profile_id: str, target_profile_id: str,
                                     limit: int = 10, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getProfileHighlights', {
            'equal': equal,
            'limit': limit,
            'sort': sort,
            'target_profile_id': target_profile_id,
            'profile_id': profile_id
        })

    async def get_bookmarked_posts(self, profile_id: str,
                                   limit: int = 50, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getBookmarkedPosts', {
            'equal': equal,
            'limit': limit,
            'sort': sort,
            'profile_id': profile_id
        })

    async def get_explore_posts(self, profile_id: str, limit: int = 50,
                                sort: str = 'FromMax', equal: bool = False, max_id: str = None) -> dict:
        return await self._execute_request('getExplorePosts', {
            'equal': equal,
            'limit': limit,
            'sort': sort,
            'max_id': max_id,
            'profile_id': profile_id
        })

    async def get_blocked_profiles(self, profile_id: str,
                                   limit: int = 50, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getBlockedProfiles', {
            'equal': equal,
            'limit': limit,
            'sort': sort,
            'profile_id': profile_id
        })

    async def get_profile_followers(self, profile_id: str, target_profile_id: str,
                                    limit: int = 50, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getProfileFollowers', {
            'equal': equal,
            'f_type': 'Follower',
            'limit': limit,
            'sort': sort,
            'target_profile_id': target_profile_id,
            'profile_id': profile_id
        })

    async def get_profile_followings(self, profile_id: str, target_profile_id: str,
                                     limit: int = 50, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getProfileFollowers', {
            'equal': equal,
            'f_type': 'Following',
            'limit': limit,
            'sort': sort,
            'target_profile_id': target_profile_id,
            'profile_id': profile_id
        })

    async def get_profile_info(self, profile_id: str, target_profile_id: str) -> dict:
        return await self._execute_request('getProfileInfo', {
            'profile_id': profile_id,
            'target_profile_id': target_profile_id
        })

    async def block_profile(self, profile_id: str, blocked_id: str) -> dict:
        return await self._execute_request('setBlockProfile', {
            'action': 'Block',
            'blocked_id': blocked_id,
            'profile_id': profile_id
        })

    async def unblock_profile(self, profile_id: str, blocked_id: str) -> dict:
        return await self._execute_request('setBlockProfile', {
            'action': 'Unblock',
            'blocked_id': blocked_id,
            'profile_id': profile_id
        })

    async def get_my_archive_stories(self, profile_id: str, limit: int = 50, sort: str = 'FromMax', equal: bool = False) -> dict:
        return await self._execute_request('getMyArchiveStories', {'equal': equal, 'limit': limit, 'sort': sort, 'profile_id': profile_id})

    async def remove_page(self, profile_id: str, record_id: str) -> dict:
        return await self._execute_request('removeRecord', {'model': 'Profile', 'record_id': record_id, 'profile_id': profile_id})

    async def upload_file(self, file: str, profile_id: str, file_type: str) -> dict:
        filename, filesize = file.split('/')[-1], os.path.getsize(file)
        result = await self.request_upload_file(profile_id, filename, filesize, file_type)
        
        byte_file = open(file, 'rb').read()
        headers = {
            'auth': self.client.auth,
            'file-id': result['data']['file_id'],
            'chunk-size': str(len(byte_file)),
            'total-part': str(1),
            'part-number': str(1),
            'hash-file-request': result['data']['hash_file_request'],
        }

        async with self.client.connection.session.post(result.server_url, data=byte_file, headers=headers) as result:
            if result.ok:
                result = await result.json()
                return Results(result.get('data')), result

# ... (continue with any other methods you have)


    async def add_post(self, profile_id: str, file: str, caption: str = None, file_type: str = 'Picture') -> dict:
        result = await self.upload_file(file, profile_id, file_type)
        return await self._execute_request('addPost', {
            'rnd': int(random() * 1e6 + 1),
            'width': 720,
            'height': 720,
            'caption': caption,
            'file_id': result[1]['file_id'],
            'post_type': file_type,
            'profile_id': profile_id,
            'hash_file_receive': result[0]['hash_file_receive'],
            'thumbnail_file_id': result[1]['file_id'],
            'thumbnail_hash_file_receive': result[0]['hash_file_receive'],
            'is_multi_file': False,
        })

    async def add_picture(self, profile_id: str, file):
        pass


import asyncio
import aiohttp
import aiorubika
import aiofiles
import json
import os
from .crypto import Crypto
from . import exceptions
from .types import Results

def capitalize(text: str):
    return ''.join([c.title() for c in text.split('_')])

class Network:
    HEADERS = {'user-agent': ('Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
                              'AppleWebKit/537.36 (KHTML, like Gecko)'
                              'Chrome/102.0.0.0 Safari/537.36'),
            	'origin': 'https://web.rubika.ir',
            	'referer': 'https://web.rubika.ir/',
                'connection': 'keep-alive',
                }

    def __init__(self, client: "aiorubika.Client") -> None:
        self.client = client
        connector = aiohttp.TCPConnector(verify_ssl=False)
        self.json_decoder = json.JSONDecoder().decode
        self.json_encoder = json.JSONEncoder().encode
        self.session = aiohttp.ClientSession(
            connector=connector,
            headers=self.HEADERS,
            timeout=aiohttp.ClientTimeout(client.timeout),
        )

        if client.bot_token is not None:
            self.bot_api_url = f'https://messengerg2b1.iranlms.ir/v3/{client.bot_token}/'

        self.api_url = None
        self.wss_url = None

    async def close(self):
        await self.session.close()

    async def get_dcs(self):
        try_count = 0

        while True:
            try:
                async with self.session.get('https://getdcmess.iranlms.ir/', verify_ssl=False) as response:
                    if not response.ok:
                        continue

                    response = (await response.json()).get('data')

                self.api_url = response.get('API').get(response.get('default_api')) + '/'
                self.wss_url = response.get('socket').get(response.get('default_socket'))
                return True

            except aiohttp.ServerTimeoutError:
                try_count += 1
                print(f'Server timeout error ({try_count})')
                await asyncio.sleep(try_count)
                continue

            except aiohttp.ClientConnectionError:
                try_count += 1
                print(f'Client connection error ({try_count})')
                await asyncio.sleep(try_count)
                continue

    async def request(self, url: str, data: dict):
        if not isinstance(data, str):
            data = self.json_encoder(data)

        if isinstance(data, str):
            data = data.encode('utf-8')

        for _ in range(3):
            try:
                async with self.session.post(url=url, data=data, verify_ssl=False) as response:
                    if response.ok:
                        return self.json_decoder(await response.text())

            except aiohttp.ServerTimeoutError:
                print('Rubika server timeout error, try again ({})'.format(_))

            except aiohttp.ClientError:
                print('Client error, try again ({})'.format(_))

            except Exception as err:
                print('Unknown Error:', err, '{}'.format(_))

    async def send(self, **kwargs):
        api_version: str = str(kwargs.get('api_version', self.client.API_VERSION))
        auth: str = kwargs.get('auth', self.client.auth)
        client: dict = kwargs.get('client', self.client.DEFAULT_PLATFORM)
        input: dict = kwargs.get('input', {})
        method: str = kwargs.get('method', 'getUserInfo')
        encrypt: bool = kwargs.get('encrypt', True)
        tmp_session: bool = kwargs.get('tmp_session', False)
        url: str = kwargs.get('url', self.api_url)

        data = dict(
            api_version=api_version,
        )

        data['tmp_session' if tmp_session is True else 'auth'] = auth if tmp_session is True else self.client.decode_auth

        if api_version == '6':
            data_enc = dict(
                client=client,
                method=method,
                input=input,
            )

            if encrypt is True:
                data['data_enc'] = Crypto.encrypt(data_enc, key=self.client.key)

            if tmp_session is False:
                data['sign'] = Crypto.sign(self.client.import_key, data['data_enc'])

            return await self.request(url, data=data)

        elif api_version == '0':
            data['auth'] = auth
            data['client'] = client
            data['data'] = input
            data['method'] = method

        elif api_version == '4':
            data['client'] = client
            data['method'] = method

        elif api_version == 'bot':
            return await self.request(
                url=self.bot_api_url + method,
                data=input,
            )

        return await self.request(url, data=data)

    async def update_handler(self, update: dict):
        if isinstance(update, str):
            update: dict = self.json_decoder(update)

        data_enc: str = update.get('data_enc')

        if data_enc:
            result = Crypto.decrypt(data_enc, key=self.client.key)
            user_guid = result.pop('user_guid')

            async def complete(name, package):
                if not isinstance(package, list):
                    return

                for update in package:
                    update['client'] = self.client
                    update['user_guid'] = user_guid

                for func, handler in self.client.handlers.items():
                    try:
                        # if handler is empty filters
                        if isinstance(handler, type):
                            handler = handler()

                        if handler.__name__ != capitalize(name):
                            return

                        # analyze handlers
                        if not await handler(update=update):
                            return

                        asyncio.create_task(func(handler))

                    except exceptions.StopHandler:
                        break

                    except Exception:
                        pass

            for name, package in result.items():
                asyncio.create_task(complete(name, package))
                        # self._client._logger.error(
                        #     'handler raised an exception', extra={'data': update}, exc_info=True)

    async def get_updates(self):
        while True:
            try:
                async with self.session.ws_connect(self.wss_url, verify_ssl=False, heartbeat=30) as ws:
                    await self.send_json_to_ws(ws)
                    asyncio.create_task(self.send_json_to_ws(ws, data=True))

                    async for msg in ws:
                        if msg.type == aiohttp.WSMsgType.TEXT:
                            asyncio.create_task(self.update_handler(msg.data))
                        elif msg.type == aiohttp.WSMsgType.CLOSED:
                            break
                        elif msg.type == aiohttp.WSMsgType.ERROR:
                            break

            except aiohttp.ClientError:
                continue

            except Exception:
                continue

    async def send_json_to_ws(self, ws: aiohttp.ClientWebSocketResponse, data=False):
        if data:
            while True:
                try:
                    await asyncio.sleep(10)
                    await ws.send_json({})
                    await self.client.get_chats_updates()
                except:
                    pass

        return await ws.send_json(dict(
            method='handShake',
            auth=self.client.auth,
            api_version='5',
            data='',
        ))

    async def upload_file(self, file, mime: str = None, file_name: str = None, chunk: int = 1048576 * 2,
                          callback=None, *args, **kwargs):
        if isinstance(file, str):
            if not os.path.exists(file):
                raise ValueError('file not found in the given path')

            if file_name is None:
                file_name = os.path.basename(file)

            async with aiofiles.open(file, 'rb') as file:
                file = await file.read()

        elif not isinstance(file, bytes):
            raise TypeError('file arg value must be file path or bytes')

        if file_name is None:
            raise ValueError('the file_name is not set')

        if mime is None:
            mime = file_name.split('.')[-1]

        result = await self.client.request_send_file(file_name, len(file), mime)

        id = result.id
        index = 0
        dc_id = result.dc_id
        total = int(len(file) / chunk + 1)
        upload_url = result.upload_url
        access_hash_send = result.access_hash_send

        while index < total:
            data = file[index * chunk: index * chunk + chunk]
            try:
                result = await self.session.post(
                        upload_url,
                        headers={
                            'auth': self.client.auth,
                            'file-id': id,
                            'total-part': str(total),
                            'part-number': str(index + 1),
                            'chunk-size': str(len(data)),
                            'access-hash-send': access_hash_send
                        },
                        data=data
                    )
                result = await result.json()
                
                if result.get('status') != 'OK':
                    raise exceptions.UploadError(result.get('status'),
                                                 result.get('status_det'),
                                                 dev_message=result.get('dev_message'))

                if callable(callback):
                    try:
                        await callback(len(file), index * chunk)

                    except exceptions.CancelledError:
                        return None

                    except Exception:
                        pass

                index += 1

            except Exception:
                   pass

        status = result['status']
        status_det = result['status_det']

        if status == 'OK' and status_det == 'OK':
            result = {
                'mime': mime,
                'size': len(file),
                'dc_id': dc_id,
                'file_id': id,
                'file_name': file_name,
                'access_hash_rec': result['data']['access_hash_rec']
            }

            return Results(result)

        #self._client._logger.debug('upload failed', extra={'data': result})
        raise exceptions(status_det)(result, request=result)
    
    async def download(self, dc_id: int, file_id: int, access_hash: str, size: int, chunk=131072, callback=None):
        url = f'https://messenger{dc_id}.iranlms.ir/GetFile.ashx'
        start_index = 0
        result = b''

        headers = {
            'auth': self.client.auth,
            'access-hash-rec': access_hash,
            'file-id': str(file_id),
            'user-agent': self.client.user_agent
        }

        async with aiohttp.ClientSession() as session:
            while True:
                last_index = start_index + chunk - 1 if start_index + chunk < size else size - 1

                headers['start-index'] = str(start_index)
                headers['last-index'] = str(last_index)

                response = await session.post(url, headers=headers)
                if response.ok:
                    data = await response.read()
                    if data:
                        result += data

                        if callback:
                            await callback(size, len(result))

                # Check for the end of the file
                if len(result) >= size:
                    break

                # Update the start_index value to fetch the next part of the file
                start_index = last_index + 1

        return result


from typing import Optional, Coroutine
from asyncio import run

import rubpy


class Run:
    def run(self: "rubpy.Client", coroutine: Optional[Coroutine] = None, phone_number: str = None):
        async def main_runner():
            await self.start(phone_number=phone_number)
            await self.connection.get_updates()

        if coroutine:
            run(coroutine)

        run(main_runner())


from typing import Union


class GetAbsObjects:
    async def get_abs_objects(
            self,
            object_guids: Union[str, list],
    ):
        if isinstance(object_guids, str):
            object_guids = list(object_guids)

        return await self.builder('getAbsObjects',
                                  input={
                                      'object_guids': object_guids,
                                  })


import rubpy

class GetProfileLinkItems:
    async def get_profile_link_items(
            self: "rubpy.Client",
            object_guid: str,
    ):
        return await self.builder('getProfileLinkItems', input=dict(object_guid=object_guid))


from typing import Union


class RequestSendFile:
    async def request_send_file(
            self,
            file_name: str,
            size: Union[str, int, float],
            mime: str=None,
    ):
        input = {
            'file_name': file_name,
            'size': int(size),
            'mime': file_name.split('.')[-1] if mime is None else mime
        }
        return await self.builder('requestSendFile',
                                  input=input)


from ...types import Results
from ..utilities import thumbnail
from typing import Optional, Union
from aiofiles import open as aiopen
from asyncio import create_task
from pathlib import Path
from os import path
from random import random
import rubpy
import aiohttp


class SendMessage:
    async def send_message(self: "rubpy.Client",
                           object_guid: str,
                           text: Optional[str] = None,
                           reply_to_message_id: Optional[str] = None,
                           file_inline: Optional[Union[Path, bytes]] = None,
                           type: str = 'File',
                           is_spoil: bool = False,
                           thumb: bool = True,
                           auto_delete: Optional[int] = None,
                           parse_mode: Optional[str] = None, *args, **kwargs):
        """_send message_

        Args:
            object_guid (str):
                _object guid_

            message (Any, optional):
                _message or cation or sticker_ . Defaults to None.

            reply_to_message_id (str, optional):
                _reply to message id_. Defaults to None.

            file_inline (typing.Union[pathlib.Path, bytes], optional):
                _file_. Defaults to None.

            type (str, optional):
                _file type_. Defaults to methods.messages.File.(
                    methods.messages.Gif,
                    methods.messages.Image,
                    methods.messages.Voice,
                    methods.messages.Music,
                    methods.messages.Video
                )

            thumb (bool, optional):
                if value is "True",
                    the lib will try to build the thumb ( require cv2 )
                if value is thumbnail.Thumbnail, to set custom
                Defaults to True.
        """
        if object_guid.lower() in ('me', 'cloud', 'self'):
            object_guid = self.guid

        input = {
            'object_guid': object_guid,
            'reply_to_message_id': reply_to_message_id,
            'rnd': int(random() * 1e6 + 1),
        }

        if isinstance(text, str):
            input['text'] = text.strip()

            markdown = self.markdown.to_metadata(text)
            if 'metadata' in markdown.keys():
                input['metadata'] = markdown.get('metadata')
                input['text'] = markdown.get('text')

        if file_inline:
            if not isinstance(file_inline, Results):
                if isinstance(file_inline, str):
                    if file_inline.startswith('http'):
                        async with aiohttp.ClientSession(headers={'user-agent': self.user_agent}) as cs:
                            async with cs.get(file_inline) as result:
                                if result.ok:
                                    file_name = file_inline.split('/')[-1]
                                    file_inline = await result.read()
                                    kwargs['file_name'] = kwargs.get('file_name',
                                                                     file_name if '.' in file_name else file_name+'.'+type)

                    else:
                        async with aiopen(file_inline, 'rb') as file:
                            kwargs['file_name'] = kwargs.get(
                                'file_name', path.basename(file_inline))
                            file_inline = await file.read()

                if type in ('Music', 'Voice'):
                    thumb = None

                if thumb:
                    if type in ('Video', 'Gif'):
                        thumb = thumbnail.MediaThumbnail.from_video(file_inline)
                    elif type == 'Image':
                        thumb = thumbnail.MediaThumbnail.from_image(file_inline)
                    elif type == 'VideoMessage':
                        thumb = thumbnail.MediaThumbnail.from_video(file_inline)

                    if not hasattr(thumb, 'image'):
                        type = 'File'
                        thumb = None

                file_inline = await self.upload(file_inline, *args, **kwargs)

                if type == 'VideoMessage':
                    file_inline['is_round'] = True

                file_inline['type'] = 'Video' if type == 'VideoMessage' else type
                file_inline['time'] = kwargs.get('time', 1)
                file_inline['width'] = kwargs.get('width', 200)
                file_inline['height'] = kwargs.get('height', 200)
                file_inline['music_performer'] = kwargs.get('performer', '')

                if isinstance(thumb, thumbnail.ResultMedia):
                    file_inline['time'] = thumb.seconds
                    file_inline['width'] = thumb.width
                    file_inline['height'] = thumb.height
                    file_inline['thumb_inline'] = thumb.to_base64()

        if file_inline:
            file_inline['is_spoil'] = bool(is_spoil)
            input['file_inline'] = file_inline.to_dict()

        result = await self.builder('sendMessage', input=input)

        if auto_delete is not None:
            if not isinstance(auto_delete, int):
                raise ValueError('The `auto_delete` parameter can only be Integer.')

            create_task(self.auto_delete_message(result.object_guid,
                                                         result.message_id,
                                                         auto_delete))

        return result
        # message = messages.SendMessage(**result.to_dict())
        # await message.set_shared_data(self, message)
        # return message


class GetGroupAdminAccessList:
    async def get_group_admin_access_list(
            self,
            group_guid: str,
            member_guid: str,
    ):
        return await self.builder('getGroupAdminAccessList',
                                  input={
                                      'group_guid': group_guid,
                                      'member_guid': member_guid,
                                  })


from typing import Union
import rubpy

class AddChannelMembers:
    async def add_channel_members(
            self: "rubpy.Client",
            channel_guid: str,
            member_guids: Union[str, list],
    ):
        if isinstance(member_guids, str):
            member_guids = [member_guids]

        return await self.builder('addChannelMembers',
                                  input={
                                      'channel_guid': channel_guid,
                                      'member_guids': member_guids,
                                  })


import rubpy

class UpdateChannelUsername:
    async def update_channel_username(
            self: "rubpy.Client",
            channel_guid: str,
            username: str,
    ):
        input = {
            'channel_guid': channel_guid,
            'username': username.replace('@', '')
        }

        return await self.builder('updateChannelUsername',
                                  input=input)


from pydantic import BaseModel
from typing import Optional, List

class AvatarThumbnail(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None

class OnlineTime(BaseModel):
    type: Optional[str] = None
    exact_time: Optional[int] = None

class LastMessage(BaseModel):
    message_id: Optional[str] = None
    type: Optional[str] = None
    text: Optional[str] = None
    author_object_guid: Optional[str] = None
    is_mine: Optional[bool] = None
    author_title: Optional[str] = None
    author_type: Optional[str] = None

class ChatInfo(BaseModel):
    object_guid: Optional[str] = None
    access: Optional[List[str]] = []
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[LastMessage] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None

class UserInfo(BaseModel):
    user_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    phone: Optional[str] = None
    username: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    last_online: Optional[int] = None
    bio: Optional[str] = None
    is_deleted: Optional[bool] = None
    is_verified: Optional[bool] = None
    online_time: Optional[OnlineTime] = None

class AvatarData(BaseModel):
    avatar_id: Optional[str] = None
    thumbnail: Optional[AvatarThumbnail] = None
    main: Optional[AvatarThumbnail] = None
    create_time: Optional[int] = None

class GetAvatars(BaseModel):
    avatars: Optional[List[AvatarData]] = []
    _client: Optional[str] = None
    original_update: Optional[str] = None

class ChatAccess(BaseModel):
    access: Optional[List[str]] = []
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[LastMessage] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None
    pinned_message_id: Optional[str] = None
    abs_object: Optional[dict] = None
    is_blocked: Optional[bool] = None
    last_message_id: Optional[str] = None
    last_deleted_mid: Optional[str] = None
    slow_mode_duration: Optional[int] = None
    group_my_last_send_time: Optional[int] = None
    pinned_message_ids: Optional[List[str]] = []

class ChatUpdate(BaseModel):
    object_guid: Optional[str] = None
    action: Optional[str] = None
    chat: Optional[ChatAccess] = None
    updated_parameters: Optional[List[str]] = []
    timestamp: Optional[str] = None
    type: Optional[str] = None

class DeleteAvatar(BaseModel):
    user: Optional[UserInfo] = None
    chat_update: Optional[ChatUpdate] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetChats(BaseModel):
    chats: Optional[List[ChatInfo]] = []
    next_start_id: Optional[str] = None
    state: Optional[int] = None
    has_continue: Optional[bool] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class SeenChats(BaseModel):
    chat_updates: Optional[List[ChatUpdate]] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class SetActionChat(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class GetChatsUpdates(BaseModel):
    chats: List[ChatInfo]
    new_state: Optional[int] = None
    status: Optional[str] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class SendChatActivity(BaseModel):
    _client: Optional[str] = None
    original_update: Optional[str] = None

class DeleteChatHistory(BaseModel):
    chat_update: Optional[ChatUpdate] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class SearchChatMessages(BaseModel):
    message_ids: List[str]
    _client: Optional[str] = None
    original_update: Optional[str] = None



import rubpy

class GetStickersByEmoji:
    async def get_stickers_by_emoji(
            self: "rubpy.Client",
            emoji: str,
            suggest_by: str = 'Add',
    ):
        return await self.builder(
            name='getStickersByEmoji',
            input={'emoji': emoji, 'suggest_by': suggest_by}
        )


from typing import Optional
import rubpy

class UpdateProfile:
    async def update_profile(
            self: "rubpy.Client",
            first_name: Optional[str] = None,
            last_name: Optional[str] = None,
            bio: Optional[str] = None,
    ):
        if first_name and last_name and bio is None:
            raise ValueError('All parameters are None.')

        input = {'updated_parameters': []}

        if first_name is not None:
            input['updated_parameters'].append('first_name')
            input['first_name'] = first_name

        if last_name is not None:
            input['updated_parameters'].append('last_name')
            input['last_name'] = last_name

        if bio is not None:
            input['updated_parameters'].append('bio')
            input['bio'] = bio

        return await self.builder(name='updateProfile', input=input)


from typing import Union


class AddGroupMembers:
    async def add_group_members(
            self,
            group_guid: str,
            member_guids: Union[str, list],
    ):
        if isinstance(member_guids, str):
            member_guids = [member_guids]

        return await self.builder('addGroupMembers',
                                  input={
                                      'group_guid': group_guid,
                                      'member_guids': member_guids,
                                  })


import rubpy

class BanMember:
    async def ban_member(
            self: "rubpy.Client",
            object_guid: str,
            member_guid: str,
    ):
        if object_guid.startswith('g0'):
            return await self.ban_group_member(object_guid, member_guid)
        elif object_guid.startswith('c0'):
            return await self.ban_channel_member(object_guid, member_guid)


from .sessions import SQLiteSession, StringSession
from .parser import Markdown
from .methods import Methods
from typing import Optional, Union


class Client(Methods):
    DEFAULT_PLATFORM = {
        'app_name': 'Main',
        'app_version': '4.4.6',
        'platform': 'Web',
        'package': 'web.rubika.ir',
        }

    USER_AGENT = ('Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
                  'AppleWebKit/537.36 (KHTML, like Gecko)'
                  'Chrome/102.0.0.0 Safari/537.36')

    API_VERSION = '6'

    def __init__(self,
                 name: str,
                 auth: Optional[str] = None,
                 private_key: Optional[Union[str, bytes]] = None,
                 bot_token: Optional[str] = None,
                 phone_number: Optional[str] = None,
                 user_agent: Optional[str] = None or USER_AGENT,
                 timeout: Optional[Union[str, int]] = 20,
                 lang_code: Optional[str] = 'fa',
                 parse_mode: Optional[str] = 'All',
    ) -> None:
        super().__init__()
        if auth and not isinstance(auth, str):
            raise ValueError('`auth` is `string` arg.')

        if private_key:
            if not type(private_key) in (str, bytes):
                raise ValueError('`private_key` is `string` or `bytes` arg.')

        if bot_token and not isinstance(bot_token, str):
            raise ValueError('`bot_token` is `string` arg.')

        if phone_number and not isinstance(phone_number, str):
            raise ValueError('`phone_number` is `string` arg.')

        if user_agent and not isinstance(user_agent, str):
            raise ValueError('`user_agent` is `string` arg.')

        if not isinstance(timeout, int):
            timeout = int(timeout)

        if isinstance(name, str):
            session = SQLiteSession(name)

        elif not isinstance(name, StringSession):
            raise TypeError('The given session must be a '
                            'str or [rubpy.sessions.StringSession]')

        if parse_mode not in ('All', 'html', 'markdown', 'mk'):
            raise ValueError('The `parse_mode` argument can only be in `("All", "html", "markdown", "mk")`.')

        self.DEFAULT_PLATFORM['lang_code'] = lang_code
        self.name = name
        self.auth = auth
        self.private_key = private_key
        self.bot_token = bot_token
        self.phone_number = phone_number
        self.user_agent = user_agent
        self.lang_code = lang_code
        self.timeout = timeout
        self.session = session
        self.parse_mode = parse_mode
        self.markdown = Markdown()
        self.database = None
        self.decode_auth = None
        self.import_key = None
        self.guid = None
        self.key = None
        self.handlers = {}

    def __enter__(self):
        return self.start()

    def __exit__(self, *args, **kwargs):
        try:
            return self.disconnect()
        except Exception as exc:
            print(exc.__name__, exc)

    async def __aenter__(self):
        return await self.start()

    async def __aexit__(self, *args, **kwargs):
        try:
            return await self.disconnect()
        except Exception as exc:
            print(exc.__name__, exc)


import rubpy
from ...types.in_chat_members import InChatMembers


class GetChannelAllMembers:
    async def get_channel_all_members(
            self: "rubpy.Client",
            channel_guid: str,
            search_text: str='',
            start_id: str=None,
    ):
        result = await self.builder('getChannelAllMembers',
                                  input={
                                      'channel_guid': channel_guid,
                                      'search_text': search_text,
                                      'start_id': start_id,
                                  }, dict=True)
        return InChatMembers(**result)


import rubpy
from typing import Union, Optional


class GetPollOptionVoters:
    async def get_poll_option_voters(
            self: "rubpy.Client",
            poll_id: str,
            selection_index: Union[str, int],
            start_id: Optional[str] = None,
    ):
        input = {'poll_id': poll_id,
                 'selection_index': selection_index,
                 'start_id': start_id,
                }

        return await self.builder(name='getPollOptionVoters', input=input)


from typing import Union


class GetMessagesByID:
    async def get_messages_by_id(
            self,
            object_guid: str,
            message_ids: Union[str, list],
    ):
        if isinstance(message_ids, str):
            message_ids = [str(message_ids)]

        return await self.builder('getMessagesByID',
                                  input={
                                      'object_guid': object_guid,
                                      'message_ids': message_ids,
                                  })


import rubpy

from typing import Literal, Optional

class ActionOnMessageReaction:
    async def action_on_message_reaction(
            self: "rubpy.Client",
            object_guid: str,
            message_id: str,
            reaction_id: int = None,
            action=Literal['Add', 'Remove'],
    ):
        input = dict(
            object_guid=object_guid,
            message_id=message_id,
            action=action,
            reaction_id=reaction_id,
        )

        return await self.builder(name='actionOnMessageReaction',
                                  input=input)


import rubpy

class GetChannelAdminMembers:
    async def get_channel_admin_members(
            self,
            channel_guid: str,
            start_id: str=None,
    ):
        return await self.builder('getChannelAdminMembers',
                                  input={
                                      'channel_guid': channel_guid,
                                      'start_id': start_id,
                                  })


from typing import Literal, List
from pydantic import BaseModel


class PollStatus(BaseModel):
    state: Literal['Open', 'Closed']
    selection_index: int
    percent_vote_options: List[int]
    total_vote: int
    show_total_votes: bool


from json import dumps
from base64 import b64decode
from typing import Literal, Union
from pathlib import Path
import rubpy
thumbnail = None


class SocketResults:
    def __str__(self) -> str:
        return self.jsonify(indent=2)

    def __getattr__(self, name):
        return self.find_keys(keys=name)

    def __setitem__(self, key, value):
        self.original_update[key] = value

    def __getitem__(self, key):
        return self.original_update[key]

    def __lts__(self, update: list, *args, **kwargs):
        for index, element in enumerate(update):
            if isinstance(element, list):
                update[index] = self.__lts__(update=element)

            elif isinstance(element, dict):
                update[index] = SocketResults(update=element)

            else:
                update[index] = element
        return update

    def __init__(self, update: dict, *args, **kwargs) -> None:
        self.client: "rubpy.Client" = update.get('client')
        self.original_update = update

    def to_dict(self):
        return self.original_update

    def jsonify(self, indent=None, *args, **kwargs) -> str:
        result = self.original_update
        result['original_update'] = 'dict{...}'
        return dumps(result, indent=indent,
                          ensure_ascii=False,
                          default=lambda value: str(value))

    def find_keys(self, keys, original_update=None, *args, **kwargs):

        if original_update is None:
            original_update = self.original_update

        if not isinstance(keys, list):
            keys = [keys]

        if isinstance(original_update, dict):
            for key in keys:
                try:
                    update = original_update[key]
                    if isinstance(update, dict):
                        update = SocketResults(update=update)

                    elif isinstance(update, list):
                        update = self.__lts__(update=update)

                    return update

                except KeyError:
                    pass
            original_update = original_update.values()

        for value in original_update:
            if isinstance(value, (dict, list)):
                try:
                    return self.find_keys(keys=keys, original_update=value)

                except AttributeError:
                    return None

        return None

    @property
    def type(self):
        try:
            return self.find_keys(keys=['type', 'author_type'])

        except AttributeError:
            pass

    @property
    def raw_text(self):
        try:
            return self.find_keys(keys='text')

        except AttributeError:
            pass

    @property
    def message_id(self):
        try:
            return self.find_keys(keys=['message_id',
                                        'pinned_message_id'])
        except AttributeError:
            pass

    @property
    def reply_message_id(self):
        try:
            return self.find_keys(keys='reply_to_message_id')

        except AttributeError:
            pass

    @property
    def is_group(self):
        return self.type == 'Group'

    @property
    def is_channel(self):
        return self.type == 'Channel'

    @property
    def is_private(self):
        return self.type == 'User'

    @property
    def object_guid(self):
        try:
            return self.find_keys(keys=['group_guid',
                                        'object_guid', 'channel_guid'])
        except AttributeError:
            pass

    @property
    def author_guid(self):
        try:
            return self.author_object_guid

        except AttributeError:
            pass

    @property
    def text(self):
        return self.message.type == 'Text'

    @classmethod
    def guids(cls, _: list) -> bool:
        if type(_) == list:
            return cls.object_guid in _
        return cls.object_guid == _

    def guid_type(self, object_guid: str):
        if object_guid.startswith('c'):
            return 'Channel'
        elif object_guid.startswith('g'):
            return 'Group'
        else:
            return 'User'

    # async methods

    async def pin(self,
                  object_guid: str = None,
                  message_id: str = None,
                  action: str = 'Pin'):
        """_pin_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_id (str, optional):
                _custom message id_. Defaults to update.message_id.
            action (bool, optional):
                _pin or unpin_. Defaults to methods.messages.Pin. (
                    methods.messages.Pin,
                    methods.messages.Unpin
                )
        Returns:
            BaseResults: result
        """

        if object_guid is None:
            object_guid = self.object_guid

        if message_id is None:
            message_id = self.message_id

        return await self.client.set_pin_message(object_guid, message_id, action)

    async def edit(self,
                   text: str,
                   object_guid: str = None,
                   message_id: str = None, *args, **kwargs):
        """_edit_
        Args:
            text (str):
                _message text_
            object_guid (str, optional):
                _custom objec guid_. Defaults to update.object_guid.
            message_id (str, optional):
                _custom message id_. Defaults to update.message_id.
        """

        if object_guid is None:
            object_guid = self.object_guid

        if message_id is None:
            message_id = self.message_id

        return await self.client.edit_message(object_guid, message_id, text)

    async def copy(self,
                   to_object_guid: str,
                   from_object_guid: str = None, message_ids=None, *args, **kwargs):
        """_copy_
        Args:
            to_object_guid (str):
                _to object guid_.
            from_object_guid (str, optional):
                _from object guid_. Defaults to update.object_guid.
            message_ids (typing.Union[str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """

        if from_object_guid is None:
            from_object_guid = self.object_guid
        
        if message_ids is None:
            message_ids = self.message_id
        
        result = await self.get_messages(from_object_guid, message_ids)
        messages = []
        if result.messages:
            for message in result.messages:
                
                try:
                    file_inline = message.file_inline
                    kwargs.update(file_inline.to_dict())

                except AttributeError:
                    file_inline = None

                try:
                    thumb = thumbnail.Thumbnail(b64decode(message.thumb_inline))
                    
                except AttributeError:
                    thumb = kwargs.get('thumb', True)
                                
                try:
                    message = message.sticker
                
                except AttributeError:
                    message = message.raw_text
                
                if file_inline is not None:
                    if file_inline.type not in ['Gif',
                                                'Sticker']:
                        file_inline = await self.download(file_inline)
                        messages.append(await self.client.send_message(
                            thumb=thumb,
                            message=message,
                            file_inline=file_inline,
                            object_guid=to_object_guid, *args, **kwargs))
                        continue

                messages.append(await self.client.send_message(
                    message=message,
                    object_guid=to_object_guid,
                    file_inline=file_inline, *args, **kwargs))
    
        return {'status': 'OK', 'messages': messages}

    async def seen(self, seen_list: dict = None):
        """_seen_
        Args:
            seen_list (dict, optional):
                _description_. Defaults t
                    {update.object_guid: update.message_id}
        """

        if seen_list is None:
            seen_list = {self.object_guid: self.message_id}
        return await self.client.seen_chats(seen_list)

    async def reply(self,
                    text: str = None,
                    object_guid: str = None,
                    reply_to_message_id: str = None,
                    file_inline = None,
                    auto_delete: int = None, *args, **kwargs,
    ):
        """_reply_
        Args:
            message (Any, optional):
                _message or cation or sticker_ . Defaults to None.
            object_guid (str):
                _object guid_ . Defaults to update.object_guid
            reply_to_message_id (str, optional):
                _reply to message id_. Defaults to None.
            file_inline (typing.Union[pathlib.Path, bytes], optional):
                _file_. Defaults to None.
            type ('Image', 'Video', 'File', 'Voice', 'Music')
                _type_. Defaults to 'File'.
            thumb (bool, optional):
                if value is "True",
                the lib will try to build the thumb ( require cv2 )
                if value is thumbnail.Thumbnail, to set custom
                Defaults to True.
        """

        if object_guid is None:
            object_guid = self.object_guid

        if reply_to_message_id is None:
            reply_to_message_id = self.message_id

        return await self.client.send_message(object_guid, text, reply_to_message_id=reply_to_message_id,
                                              file_inline=file_inline,
                                              auto_delete=auto_delete,
                                              *args, **kwargs,)

    async def reply_document(
            self,
            document: Union[str, bytes, Path],
            caption: str = None,
            auto_delete: int = None,
            file_name: str = None,
            object_guid: str = None,
            reply_to_message_id: str = None, *args, **kwargs,
    ):
        return await self.reply(
            text=caption,
            object_guid=object_guid,
            reply_to_message_id=reply_to_message_id,
            file_inline=document,
            file_name=file_name,
            type='File',
            auto_delete=auto_delete, *args, **kwargs,
        )

    async def reply_photo(
            self,
            photo: Union[str, bytes, Path],
            caption: str = None,
            auto_delete: int = None,
            file_name: str = None,
            object_guid: str = None,
            reply_to_message_id: str = None, *args, **kwargs,
    ):
        return await self.reply(
            text=caption,
            object_guid=object_guid,
            reply_to_message_id=reply_to_message_id,
            file_inline=photo,
            file_name=file_name,
            type='Image',
            auto_delete=auto_delete, *args, **kwargs,
        )

    async def reply_video(
            self,
            video: Union[str, bytes, Path],
            caption: str = None,
            auto_delete: int = None,
            file_name: str = None,
            object_guid: str = None,
            reply_to_message_id: str = None, *args, **kwargs,
    ):
        return await self.reply(
            text=caption,
            object_guid=object_guid,
            reply_to_message_id=reply_to_message_id,
            file_inline=video,
            file_name=file_name,
            type='Video',
            auto_delete=auto_delete, *args, **kwargs,
        )
    
    async def reply_music(
            self,
            video: Union[str, bytes, Path],
            caption: str = None,
            auto_delete: int = None,
            file_name: str = None,
            object_guid: str = None,
            reply_to_message_id: str = None, *args, **kwargs,
    ):
        return await self.reply(
            text=caption,
            object_guid=object_guid,
            reply_to_message_id=reply_to_message_id,
            file_inline=video,
            file_name=file_name,
            type='Music',
            auto_delete=auto_delete, *args, **kwargs,
        )

    async def reply_voice(
            self,
            video: Union[str, bytes, Path],
            caption: str = None,
            auto_delete: int = None,
            file_name: str = None,
            object_guid: str = None,
            reply_to_message_id: str = None, *args, **kwargs,
    ):
        return await self.reply(
            text=caption,
            object_guid=object_guid,
            reply_to_message_id=reply_to_message_id,
            file_inline=video,
            file_name=file_name,
            type='Voice',
            auto_delete=auto_delete, *args, **kwargs,
        )

    async def reply_gif(
            self,
            gif: Union[str, bytes, Path],
            caption: str = None,
            auto_delete: int = None,
            file_name: str = None,
            object_guid: str = None,
            reply_to_message_id: str = None, *args, **kwargs,
    ):
        return await self.reply(
            text=caption,
            object_guid=object_guid,
            reply_to_message_id=reply_to_message_id,
            file_inline=gif,
            file_name=file_name,
            type='Gif',
            auto_delete=auto_delete, *args, **kwargs,
        )

    async def reply_video_message(
            self,
            video: Union[str, bytes, Path],
            caption: str = None,
            auto_delete: int = None,
            file_name: str = None,
            object_guid: str = None,
            reply_to_message_id: str = None, *args, **kwargs,
    ):
        return await self.reply(
            text=caption,
            object_guid=object_guid,
            reply_to_message_id=reply_to_message_id,
            file_inline=video,
            file_name=file_name,
            type='VideoMessage',
            auto_delete=auto_delete, *args, **kwargs,
        )

    async def forwards(self,
                       to_object_guid: str,
                       from_object_guid: str = None,
                       message_ids=None, *args, **kwargs):
        """_forwards_
        Args:
            to_object_guid (str):
                _to object guid_.
            from_object_guid (str, optional):
                _from object guid_. Defaults to update.object_guid.
            message_ids (typing.Union[str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """

        if from_object_guid is None:
            from_object_guid = self.object_guid

        if message_ids is None:
            message_ids = self.message_id

        return await self.client.forward_messages(
            from_object_guid=from_object_guid,
            to_object_guid=to_object_guid,
            message_ids=message_ids,
        )

    async def download(self, file_inline=None, file=None, *args, **kwargs):
        return await self._client.download_file_inline(
            file_inline or self.file_inline,
            file=file, *args, **kwargs)
    
    async def get_author(self, author_guid: str = None, *args, **kwargs):
        """_get user or author information_
        Args:
            author_guid (str, optional):
                _custom author guid_. Defaults to update.author_guid
        """

        if author_guid is None:
            author_guid = self.author_guid

        return await self.get_object(object_guid=author_guid, *args, **kwargs)

    async def get_object(self, object_guid: str = None, *args, **kwargs):
        """_get object information_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
        """

        if object_guid is None:
            object_guid = self.object_guid

        if self.guid_type(object_guid) == 'User':
            return await self.client.get_user_info(object_guid)

        elif self.guid_type(object_guid) == 'Group':
            return await self.client.get_group_info(object_guid)

        elif self.guid_type(object_guid) == 'Channel':
            return await self.client.get_channel_info(object_guid)

    async def get_messages(self,
                           object_guid: str = None,
                           message_ids=None, *args, **kwargs):
        """_get messages_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_ids (str, int, typing.List[str]], optional):
                _message ids_. Defaults to update.message_id.
        """

        if object_guid is None:
            object_guid = self.object_guid

        if message_ids is None:
            message_ids = self.message_id

        return await self.client.get_messages_by_id(
            object_guid=object_guid,
            message_ids=message_ids,
        )

    async def delete_messages(
            self,
            object_guid: str=None,
            message_ids: list=None
    ):
        """_delete messages_
        Args:
            object_guid (str, optional):
                _custom object guid_. Defaults to update.object_guid.
            message_ids (str, list, optional):
                _custom message ids_. Defaults to update.message_id.
            type(str, optional):
                the message should be deleted for everyone or not.
                Defaults to methods.messages.Global (
                    methods.messages.Local,
                    methods.messages.Global
                )
        """

        if object_guid is None:
            object_guid = self.object_guid

        if message_ids is None:
            message_ids = self.message_id

        return await self.client.delete_messages(
            object_guid=object_guid,
            message_ids=message_ids,
        )

    async def reaction(
            self,
            reaction_id: int,
            object_guid: str = None,
            message_id: str = None,
    ):
        if object_guid is None:
            object_guid = self.object_guid

        if message_id is None:
            message_id = self.message_id

        return await self.client.action_on_message_reaction(
            object_guid=object_guid,
            message_id=message_id,
            action='Add',
            reaction_id=reaction_id,
        )

    async def ban_member(
            self,
            object_guid: str = None,
            user_guid: str = None,
    ):
        if object_guid is None:
            object_guid = self.object_guid

        if object_guid.startswith('g0'):
            return await self.client.ban_group_member(object_guid, user_guid)

        elif object_guid.startswith('c0'):
            return await self.client.ban_channel_member(object_guid, user_guid)

    async def unban_member(
            self,
            object_guid: str = None,
            user_guid: str = None,
    ):
        if object_guid is None:
            object_guid = self.object_guid

        if object_guid.startswith('g0'):
            return await self.client.ban_group_member(object_guid, user_guid, 'Unset')

        elif object_guid.startswith('c0'):
            return await self.client.ban_channel_member(object_guid, user_guid, 'Unset')

    async def send_activity(
            self,
            activity: Literal['Typing', 'Uploading', 'Recording'],
            object_guid: str = None,
    ):
        if object_guid is None:
            object_guid = self.object_guid

        return await self.client.send_chat_activity(object_guid=object_guid,
                                                    activity=activity)


from pydantic import BaseModel
from typing import Optional, List

class OnlineTime(BaseModel):
    type: Optional[str] = None
    approximate_period: Optional[str] = None

class AvatarThumbnail(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None
    file_name: Optional[str] = None
    cdn_tag: Optional[str] = None

class WarningInfo(BaseModel):
    warning_id: Optional[str] = None
    title: Optional[str] = None
    text: Optional[str] = None
    link: Optional[dict] = None
    title_color: Optional[dict] = None

class User(BaseModel):
    user_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    phone: Optional[str] = None
    username: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    last_online: Optional[int] = None
    bio: Optional[str] = None
    is_deleted: Optional[bool] = None
    is_verified: Optional[bool] = None
    online_time: Optional[OnlineTime] = None
    warning_info: Optional[WarningInfo] = None

class DeleteContact(BaseModel):
    user: Optional[User] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class AddAddressBook(BaseModel):
    user: Optional[User] = None
    user_exist: Optional[bool] = None
    timestamp: Optional[str] = None
    can_receive_call: Optional[bool] = None
    can_video_call: Optional[bool] = None

class GetContacts(BaseModel):
    users: Optional[List[User]] = []
    next_start_id: Optional[str] = None
    has_continue: Optional[bool] = None
    state: Optional[int] = None
    timestamp: Optional[str] = None


from .advanced import Advanced
from .utilities import Utilities
from .users import Users
from .auth import Auth
from .messages import Messages
from .chats import Chats
from .groups import Groups
from .decorators import Decorators
from .contacts import Contacts
from .settings import Settings
from .stickers import Stickers
from .channels import Channels
from .extras import Exctras
from .gif import Gifs


class Methods(
    Advanced,
    Utilities,
    Users,
    Auth,
    Messages,
    Chats,
    Groups,
    Decorators,
    Contacts,
    Settings,
    Stickers,
    Channels,
    Exctras,
    Gifs,
):
    pass


import rubpy

class GetInfo:
    async def get_info(
            self: "rubpy.Client",
            object_guid: str = None,
            username: str = None,
    ):
        if isinstance(object_guid, str):
            if object_guid.startswith('c0'):
                return await self.get_channel_info(object_guid)
            elif object_guid.startswith('u0'):
                return await self.get_user_info(object_guid)
            elif object_guid.startswith('g0'):
                return await self.get_group_info(object_guid)

        if isinstance(username, str):
            return await self.get_object_by_username(username)


class SeenChats:
    async def seen_chats(
            self,
            seen_list: dict,
    ):
        return await self.builder('seenChats',
                                  input={
                                      'seen_list': seen_list,
                                  })


import rubpy
from time import time


class GetMessagesUpdates:
    async def get_messages_updates(
            self: "rubpy.Client",
            object_guid: str,
            state: int = round(time()) - 150,
    ):
        return await self.builder('getMessagesUpdates',
                                  input=dict(
                                      object_guid=object_guid,
                                      state=int(state),
                                  ))


from typing import Optional
from pydantic import BaseModel


class FileInline(BaseModel):
    file_id: Optional[int] = None
    mime: Optional[str] = None
    dc_id: Optional[int] = None
    access_hash_rec: Optional[str] = None
    file_name: Optional[str] = None
    thumb_inline: Optional[str] = None
    width: Optional[int] = None
    height: Optional[int] = None
    time: Optional[int] = None
    size: Optional[int] = None
    type: Optional[str] = None
    is_round: Optional[bool] = None
    music_performer: Optional[str] = None


import rubpy

from pathlib import Path
from typing import Union

class UploadAvatar:
    async def upload_avatar(
            self: "rubpy.Client",
            object_guid: str,
            image: Union[Path, bytes], *args, **kwargs,
    ):
        if object_guid.lower() in ('me', 'cloud', 'self'):
            object_guid = self.guid

        if isinstance(image, str):
            kwargs['file_name'] = kwargs.get('file_name', image.split('/')[-1])
        else:
            kwargs['file_name'] = kwargs.get('file_name', 'rubpy.jpg')

        upload = await self.upload(image, *args, **kwargs)

        input = dict(
            object_guid=object_guid,
            thumbnail_file_id=upload.file_id,
            main_file_id=upload.file_id,
        )

        return await self.builder(name='uploadAvatar', input=input)


from typing import Optional, Union
from time import time


class GetChatsUpdates:
    async def get_chats_updates(
            self,
            state: Optional[Union[str, int]] = None,
    ):
        if state is None:
            state = round(time()) - 150

        else:
            state = int(state)

        return await self.builder('getChatsUpdates',
                                  input={'state': state})